from decimal import Decimal
from typing import Any, Callable, List, Optional, Type, TypeVar, Dict
from uuid import UUID
import simplejson as json
from notbank_python_sdk.client_connection import ClientConnection, RequestType
from notbank_python_sdk.core.endpoint_category import EndpointCategory
from notbank_python_sdk.core.endpoints import Endpoints, WebSocketEndpoint
from notbank_python_sdk.core.converter import from_json_str, to_dict, from_dict, to_nb_dict
from notbank_python_sdk.error import ErrorCode, NotbankException
from notbank_python_sdk.models.cbu_owner import CbuOwner
from notbank_python_sdk.models.account_fee import AccountFee
from notbank_python_sdk.models.authenticate_response import AuthenticateResponse
from notbank_python_sdk.models.bank import Banks
from notbank_python_sdk.models.address import Address
from notbank_python_sdk.models.bank_account import BankAccount, BankAccounts
from notbank_python_sdk.models.cancel_order_reject_event import CancelOrderRejectEvent
from notbank_python_sdk.models.cancel_replace_order_request import CancelReplaceOrderResponse
from notbank_python_sdk.models.deposit_event import DepositEvent
from notbank_python_sdk.models.oms_fee import OmsFee
from notbank_python_sdk.models.id_response import IdResponse
from notbank_python_sdk.models.document import Document
from notbank_python_sdk.models.document_slice import DocumentSlice
from notbank_python_sdk.models.enums import EnumsResponse
from notbank_python_sdk.models.activity_report import ActivityReport
from notbank_python_sdk.models.account_trade import AccountTrade
from notbank_python_sdk.models.quote import Quote
from notbank_python_sdk.models.transaction import Transactions
from notbank_python_sdk.models.url_response import UrlResponse
from notbank_python_sdk.models.uuid_response import UuidResponse
from notbank_python_sdk.models.withdraw_tickets import WithdrawTicket
from notbank_python_sdk.models.deposit_ticket import DepositTicket
from notbank_python_sdk.models.instrument import Instrument
from notbank_python_sdk.models.level2_ticker_snapshot import Level2TickerSnapshot
from notbank_python_sdk.models.public_trade import PublicTrade, public_trade_list_from_json_str
from notbank_python_sdk.models.level1_ticker_summary import Level1TickerSummary
from notbank_python_sdk.models.level1_ticker_summary_min import Level1TickerSummaryMin, level1_ticker_summary_min_list_from_json_list_str
from notbank_python_sdk.models.order_fee import OrderFee
from notbank_python_sdk.models.ticker import Ticker, ticker_list_from_json_str
from notbank_python_sdk.models.user_info import UserInfo
from notbank_python_sdk.models.user_report_tickets import UserReportTicket
from notbank_python_sdk.models.report_writer_result_records import ReportWriterResultRecords
from notbank_python_sdk.models.order import Order
from notbank_python_sdk.models.ticker import Ticker
from notbank_python_sdk.models.fee import Fee
from notbank_python_sdk.models.level1_ticker import Level1
from notbank_python_sdk.models.level2_ticker_feed import Level2Feed, level_2_ticker_feed_list_from_json_str
from notbank_python_sdk.models.order_book import OrderBook, OrderBookRaw, order_book_from_raw
from notbank_python_sdk.models.pong import Pong
from notbank_python_sdk.models.send_order import SendOrderResponse
from notbank_python_sdk.models.currency_network_templates import CurrencyNetworkTemplates
from notbank_python_sdk.models.instrument_summary import InstrumentSummary
from notbank_python_sdk.models.ticker_summary import TickerSummary
from notbank_python_sdk.models.trade_summary import TradeSummary
from notbank_python_sdk.models.user_devices import UserDevice
from notbank_python_sdk.models.verification_level_config import InstrumentVerificationLevelConfig, ProductVerificationLevelConfig
from notbank_python_sdk.models.web_authenticate_user import WebAuthenticateUser
from notbank_python_sdk.models.account_transaction import AccountTransaction
from notbank_python_sdk.models.account_positions import AccountPosition
from notbank_python_sdk.models.instrument_statistic import InstrumentStatistic
from notbank_python_sdk.models.account_info import AccountInfo
from notbank_python_sdk.models.product import Product
from notbank_python_sdk.models.withdrawal_id_response import WithdrawalIdResponse
from notbank_python_sdk.parsing import build_subscription_handler, parse_response_fn, parse_response_list_fn
from notbank_python_sdk.requests_models.add_whitelisted_address_request import AddWhitelistedAddressRequest
from notbank_python_sdk.requests_models.authenticate_request import AuthenticateRequest
from notbank_python_sdk.requests_models.cancel_all_orders import CancelAllOrdersRequest
from notbank_python_sdk.requests_models.cancel_order import CancelOrder
from notbank_python_sdk.requests_models.cancel_order_request import CancelOrderRequest
from notbank_python_sdk.requests_models.cancel_replace_order_request import CancelReplaceOrderRequest
from notbank_python_sdk.requests_models.cancel_user_report import CancelUserReportRequest
from notbank_python_sdk.requests_models.confirm_fiat_withdraw_request import ConfirmFiatWithdrawRequest, ConfirmFiatWithdrawRequestInternal
from notbank_python_sdk.requests_models.confirm_whitelisted_address_request import ConfirmWhiteListedAddressRequest, ConfirmWhiteListedAddressRequestInternal
from notbank_python_sdk.requests_models.create_direct_quote_request import CreateDirectQuoteRequest
from notbank_python_sdk.requests_models.create_fiat_deposit_request import CreateFiatDepositRequest
from notbank_python_sdk.requests_models.create_fiat_withdraw_request import CreateFiatWithdrawRequest
from notbank_python_sdk.requests_models.create_inverse_quote_request import CreateInverseQuoteRequest
from notbank_python_sdk.requests_models.delete_whitelisted_address_request import DeleteWhiteListedAddressRequest, DeleteWhiteListedAddressRequestInternal
from notbank_python_sdk.requests_models.delete_client_bank_account_request import DeleteClientBankAccountRequest
from notbank_python_sdk.requests_models.download_document import DownloadDocumentRequest
from notbank_python_sdk.requests_models.download_document_slice import DownloadDocumentSliceRequest
from notbank_python_sdk.requests_models.execute_quote_request import ExecuteQuoteRequest
from notbank_python_sdk.requests_models.generate_pnl_activity_report import GeneratePnlActivityReportRequest
from notbank_python_sdk.requests_models.generate_product_delta_activity_report import GenerateProductDeltaActivityReportRequest
from notbank_python_sdk.requests_models.generate_trade_activity_report import GenerateTradeActivityReportRequest
from notbank_python_sdk.requests_models.generate_transaction_activity_report import GenerateTransactionActivityReportRequest
from notbank_python_sdk.requests_models.get_account_trades import GetAccountTradesRequest
from notbank_python_sdk.requests_models.fee_request import FeeRequest
from notbank_python_sdk.requests_models.add_client_bank_account_request import AddClientBankAccountRequest
from notbank_python_sdk.requests_models.get_client_bank_account_request import GetClientBankAccountRequest
from notbank_python_sdk.requests_models.get_client_bank_accounts_request import GetClientBankAccountsRequest
from notbank_python_sdk.requests_models.get_banks_request import GetBanksRequest
from notbank_python_sdk.requests_models.deposit_address_request import DepositAddressRequest
from notbank_python_sdk.requests_models.get_instrument_request import GetInstrumentRequest
from notbank_python_sdk.requests_models.get_network_templates_request import GetNetworksTemplatesRequest
from notbank_python_sdk.requests_models.get_owners_fiat_withdraw import GetOwnersFiatWithdrawRequest
from notbank_python_sdk.requests_models.get_quote_request import GetQuoteRequest
from notbank_python_sdk.requests_models.get_quotes_request import GetQuotesRequest
from notbank_python_sdk.requests_models.get_transactions_request import GetTransactionsRequest
from notbank_python_sdk.requests_models.resend_verification_code_whitelisted_address_request import ResendVerificationCodeWhitelistedAddress, ResendVerificationCodeWhitelistedAddressInternal
from notbank_python_sdk.requests_models.transfer_funds_request import TransferFundsRequest
from notbank_python_sdk.requests_models.update_one_step_withdraw_request import UpdateOneStepWithdrawRequest
from notbank_python_sdk.requests_models.verification_level_config_request import VerificationLevelConfigRequest
from notbank_python_sdk.requests_models.get_instruments_request import GetInstrumentsRequest
from notbank_python_sdk.requests_models.get_l2_snapshot import GetL2SnapshotRequest
from notbank_python_sdk.requests_models.get_last_trades import GetLastTradesRequest
from notbank_python_sdk.requests_models.get_level1_summary import GetLevel1SummaryRequest
from notbank_python_sdk.requests_models.get_level1_summary_min import GetLevel1SummaryMinRequest
from notbank_python_sdk.requests_models.get_open_orders import GetOpenOrdersRequest
from notbank_python_sdk.requests_models.get_open_trade_reports import GetOpenTradeReportsRequest
from notbank_python_sdk.requests_models.get_order_fee_request import GetOrderFeeRequest
from notbank_python_sdk.requests_models.get_order_history import GetOrderHistoryRequest
from notbank_python_sdk.requests_models.get_order_history_by_order_id import GetOrderHistoryByOrderIdRequest
from notbank_python_sdk.requests_models.get_order_status import GetOrderStatusRequest
from notbank_python_sdk.requests_models.get_orders import GetOrdersRequest
from notbank_python_sdk.requests_models.get_orders_history import GetOrdersHistoryRequest
from notbank_python_sdk.requests_models.get_ticker_history import GetTickerHistoryRequest
from notbank_python_sdk.requests_models.get_trades_history import GetTradesHistoryRequest
from notbank_python_sdk.requests_models.create_crypto_withdraw_request import CreateCryptoWithdrawRequest
from notbank_python_sdk.requests_models.get_user_info import GetUserInfoRequest
from notbank_python_sdk.requests_models.get_user_permissions import GetUserPermissionsRequest
from notbank_python_sdk.requests_models.get_user_report_tickets import GetUserReportTicketsRequest
from notbank_python_sdk.requests_models.get_user_report_tickets_by_status import GetUserReportTicketsByStatusRequest, convert_to_get_user_report_tickets_by_status_request_internal
from notbank_python_sdk.requests_models.get_user_report_writer_result_records import GetUserReportWriterResultRecordsRequest
from notbank_python_sdk.requests_models.level1 import GetLevel1Request
from notbank_python_sdk.requests_models.modify_order import ModifyOrderRequest
from notbank_python_sdk.requests_models.order_book import OrderBookRequest
from notbank_python_sdk.requests_models.get_whitelisted_addresses_request import GetWhitelistedAddressesRequest
from notbank_python_sdk.requests_models.remove_user_report_ticket import RemoveUserReportTicketRequest
from notbank_python_sdk.requests_models.schedule_product_delta_activity_report import ScheduleProductDeltaActivityReportRequest
from notbank_python_sdk.requests_models.schedule_profit_and_loss_activity_report import ScheduleProfitAndLossActivityReportRequest
from notbank_python_sdk.requests_models.schedule_trade_activity_report import ScheduleTradeActivityReportRequest
from notbank_python_sdk.requests_models.schedule_transaction_activity_report import ScheduleTransactionActivityReportRequest
from notbank_python_sdk.requests_models.subscribe_account_events_request import SubscribeAccountEventsRequest
from notbank_python_sdk.requests_models.subscribe_level1_request import SubscribeLevel1Request
from notbank_python_sdk.requests_models.subscribe_level2_request import SubscribeLevel2Request
from notbank_python_sdk.requests_models.subscribe_order_state_events_request import SubscribeOrderStateEventsRequest
from notbank_python_sdk.requests_models.subscribe_ticker_request import SubscribeTickerRequest
from notbank_python_sdk.requests_models.subscribe_trades_request import SubscribeTradesRequest
from notbank_python_sdk.requests_models.unsubscribe_account_events_request import UnsubscribeAccountEventsRequest
from notbank_python_sdk.requests_models.unsubscribe_level1_request import UnsubscribeLevel1Request
from notbank_python_sdk.requests_models.unsubscribe_level2_request import UnsubscribeLevel2Request
from notbank_python_sdk.requests_models.unsubscribe_order_state_events_request import UnsubscribeOrderStateEventsRequest
from notbank_python_sdk.requests_models.unsubscribe_ticker_request import UnsubscribeTickerRequest
from notbank_python_sdk.requests_models.unsubscribe_trades_request import UnsubscribeTradesRequest
from notbank_python_sdk.requests_models.send_order import SendOrderRequest, SendOrderRequestInternal
from notbank_python_sdk.requests_models.trades import TradesRequest
from notbank_python_sdk.requests_models.user_accounts import GetUserAccountsRequest
from notbank_python_sdk.requests_models.user_devices import GetUserDevicesRequest
from notbank_python_sdk.requests_models.account_fees_request import AccountFeesRequest
from notbank_python_sdk.requests_models.oms_fees_request import OmsFeesRequest
from notbank_python_sdk.requests_models.web_authenticate_user_request import WebAuthenticateUserRequest
from notbank_python_sdk.requests_models.get_account_transactions_request import GetAccountTransactionsRequest
from notbank_python_sdk.requests_models.get_account_positions_request import GetAccountPositionsRequest
from notbank_python_sdk.requests_models.get_account_instrument_statistics_request import GetAccountInstrumentStatisticsRequest
from notbank_python_sdk.requests_models.get_account_info_request import GetAccountInfoRequest
from notbank_python_sdk.requests_models.get_product_request import GetProductRequest
from notbank_python_sdk.requests_models.get_products_request import GetProductsRequest
from notbank_python_sdk.notbank_client_cache import NotbankClientCache
from notbank_python_sdk.websocket.callback_identifier import CallbackIdentifier
from notbank_python_sdk.websocket.subscription import Subscription, Unsubscription
from notbank_python_sdk.websocket.subscription_handler import Callback


T1 = TypeVar('T1')
T2 = TypeVar('T2')


class NotbankClient:
    _client_connection: ClientConnection
    _notbank_client_cache: NotbankClientCache

    def __init__(self, client_connection: ClientConnection):
        self._client_connection = client_connection
        self._notbank_client_cache = NotbankClientCache()

    def _get_ap_data_list(self, endpoint: str, request_data: Any, response_cls: Type[T2], no_pascal_case: List[str] = []) -> List[T2]:
        request_data_dict = to_dict(request_data)
        return self._client_connection.request(
            endpoint,
            EndpointCategory.AP,
            request_data_dict,
            parse_response_list_fn(response_cls, no_pascal_case))

    def _get_nb_data_list(self, endpoint: str, request_data: Any, response_cls: Type[T2], no_pascal_case: List[str] = []) -> List[T2]:
        request_data_dict = to_nb_dict(request_data)
        return self._client_connection.request(
            endpoint,
            EndpointCategory.NB,
            request_data_dict,
            parse_response_list_fn(response_cls, no_pascal_case, from_pascal_case=False))

    def _get_data(self, endpoint: str, request_data: Any, response_cls: Type[T2], no_pascal_case: List[str] = [], response_conversion_overrides: Dict[str, str] = {}, endpoint_category: EndpointCategory = EndpointCategory.AP) -> T2:
        request_data_dict = to_dict(request_data)
        return self._client_connection.request(
            endpoint, endpoint_category, request_data_dict, parse_response_fn(response_cls, no_pascal_case, overrides=response_conversion_overrides))

    def _get_nb_data(self, endpoint: str, request_data: Any, response_cls: Type[T2], no_pascal_case: List[str] = [], endpoint_category: EndpointCategory = EndpointCategory.NB) -> T2:
        request_data_dict = to_nb_dict(request_data)
        return self._client_connection.request(
            endpoint, endpoint_category, request_data_dict, parse_response_fn(response_cls, no_pascal_case, from_pascal_case=False))

    def _do_request(self, endpoint: str, request_data: Any, endpoint_category: EndpointCategory = EndpointCategory.AP) -> None:
        request_data_dict = to_dict(request_data)
        return self._client_connection.request(endpoint, endpoint_category, request_data_dict, lambda x: None)

    def _subscribe(self, endpoint: str, request_data: Any, callbacks: List[Callback], parse_response_fn: Callable[[Any], T2]) -> T2:
        request_data_dict = to_dict(request_data)
        return self._client_connection.subscribe(Subscription(endpoint, request_data_dict, callbacks, parse_response_fn))

    def _unsubscribe(self, endpoint: str, request_data: Any, callbacks_ids: List[str], parse_response_fn: Callable[[Any], T2]) -> T2:
        request_data_dict = to_dict(request_data)
        return self._client_connection.unsubscribe(Unsubscription(endpoint, request_data_dict, callbacks_ids, parse_response_fn))

    def close(self):
        return self._client_connection.close()

    def authenticate(self, authenticate_request: AuthenticateRequest) -> AuthenticateResponse:
        """https://apidoc.notbank.exchange/#authenticate"""
        return self._client_connection.authenticate(authenticate_request)

    def get_oms_deposit_fees(self, deposit_fees_request: OmsFeesRequest) -> List[OmsFee]:
        """
        https://apidoc.notbank.exchange/#getomsdepositfees
        """
        return self._get_ap_data_list(Endpoints.GET_OMS_DEPOSIT_FEES, deposit_fees_request, OmsFee)

    def get_oms_withdraw_fees(self, request: OmsFeesRequest) -> List[OmsFee]:
        """
        https://apidoc.notbank.exchange/#getomswithdrawfees
        """
        return self._get_ap_data_list(
            Endpoints.GET_OMS_WITHDRAW_FEES,
            request,
            OmsFee
        )

    def get_account_fees(self, account_fees_request: AccountFeesRequest) -> List[AccountFee]:
        """
        https://apidoc.notbank.exchange/#getaccountfees
        """
        return self._get_ap_data_list(Endpoints.GET_ACCOUNT_FEES, account_fees_request, AccountFee)

    def web_authenticate_user(self, web_authenticate_user_request: WebAuthenticateUserRequest) -> WebAuthenticateUser:
        """
        https://apidoc.notbank.exchange/#webauthenticateuser
        """
        return self._get_data(Endpoints.WEB_AUTHENTICATE_USER, web_authenticate_user_request, WebAuthenticateUser)

    def get_account_transactions(
        self, request: GetAccountTransactionsRequest
    ) -> List[AccountTransaction]:
        """
        https://apidoc.notbank.exchange/#getaccounttransactions
        """
        return self._get_ap_data_list(Endpoints.GET_ACCOUNT_TRANSACTIONS, request, AccountTransaction)

    def get_account_positions(self, request: GetAccountPositionsRequest) -> List[AccountPosition]:
        """
        https://apidoc.notbank.exchange/#getaccountpositions
        """
        return self._get_ap_data_list(
            Endpoints.GET_ACCOUNT_POSITIONS,
            request,
            AccountPosition
        )

    def get_account_instrument_statistics(
        self,
        request: GetAccountInstrumentStatisticsRequest
    ) -> List[InstrumentStatistic]:
        """
        https://apidoc.notbank.exchange/#getaccountinstrumentstatistics
        """
        return self._get_ap_data_list(
            Endpoints.GET_ACCOUNT_INSTRUMENT_STATISTICS,
            request,
            InstrumentStatistic
        )

    def get_account_info(self, request: GetAccountInfoRequest) -> AccountInfo:
        """
        https://apidoc.notbank.exchange/#getaccountinfo
        """
        return self._get_data(
            Endpoints.GET_ACCOUNT_INFO,
            request,
            AccountInfo,
            response_conversion_overrides={"oms_id": "OMSID"}
        )

    def logout(self) -> None:
        """
        https://apidoc.notbank.exchange/#logout
        """
        return self._do_request(
            Endpoints.LOGOUT,
            None,  # No se requiere payload en la solicitud.
        )

    def get_withdraw_fee(
        self,
        request: FeeRequest,
    ) -> Fee:
        """
        https://apidoc.notbank.exchange/#getwithdrawfee
        """
        return self._get_data(
            Endpoints.GET_WITHDRAW_FEE,
            request,
            Fee,
        )

    def get_deposit_fee(
        self,
        request: FeeRequest,
    ) -> Fee:
        """
        https://apidoc.notbank.exchange/#getdepositfee
        """
        return self._get_data(
            Endpoints.GET_DEPOSIT_FEE,
            request,
            Fee,
        )

    def get_product(
        self,
        request: GetProductRequest,
    ) -> Product:
        """
        https://apidoc.notbank.exchange/#product
        """
        return self._get_data(
            Endpoints.GET_PRODUCT,
            request,
            Product,
        )

    def get_products(
        self,
    ) -> List[Product]:
        """
        https://apidoc.notbank.exchange/#getproducts
        """
        return self._get_ap_data_list(
            Endpoints.GET_PRODUCTS,
            GetProductsRequest(),
            Product,
        )

    def get_product_by_symbol(self, symbol: str) -> Product:
        product = self._notbank_client_cache.get_product(symbol)
        if product is not None:
            return product
        products = self.get_products()
        self._notbank_client_cache.update_products(products)
        product = self._notbank_client_cache.get_product(symbol)
        if product is not None:
            return product
        raise NotbankException(
            ErrorCode.RESOURCE_NOT_FOUND,
            "no instrument found for symbol: {}".format(symbol))

    def get_product_id_by_symbol(self, symbol: str) -> int:
        return self.get_product_by_symbol(symbol).product_id

    def get_verification_level_config(
        self,
        request: VerificationLevelConfigRequest,
    ) -> ProductVerificationLevelConfig:
        """
        https://apidoc.notbank.exchange/#getverificationlevelconfig
        """
        return self._get_data(
            Endpoints.GET_VERIFICATION_LEVEL_CONFIG,
            request,
            ProductVerificationLevelConfig,
        )

    def get_instrument(
        self,
        request: GetInstrumentRequest,
    ) -> Instrument:
        """
        https://apidoc.notbank.exchange/#getinstrument
        """
        return self._get_data(
            Endpoints.GET_INSTRUMENT,
            request,
            Instrument,
        )

    def get_instruments(
        self,
        request: GetInstrumentsRequest,
    ) -> List[Instrument]:
        """
        https://apidoc.notbank.exchange/#getinstruments
        """
        return self._get_ap_data_list(
            Endpoints.GET_INSTRUMENTS,
            request,
            Instrument,
        )

    def get_instrument_by_symbol(self, symbol: str) -> Instrument:
        instrument = self._notbank_client_cache.get_instrument(symbol)
        if instrument is not None:
            return instrument
        instruments = self.get_instruments(GetInstrumentsRequest())
        self._notbank_client_cache.update_instruments(instruments)
        instrument = self._notbank_client_cache.get_instrument(symbol)
        if instrument is not None:
            return instrument
        raise NotbankException(
            ErrorCode.RESOURCE_NOT_FOUND,
            "no instrument found for symbol: {}".format(symbol))

    def get_instrument_id_by_symbol(
        self,
        symbol: str
    ) -> int:
        return self.get_instrument_by_symbol(symbol).instrument_id

    def get_instrument_verification_level_config(
        self,
        request: VerificationLevelConfigRequest,
    ) -> InstrumentVerificationLevelConfig:
        """
        https://apidoc.notbank.exchange/#getinstrumentverificationlevelconfig
        """
        return self._get_data(
            Endpoints.GET_INSTRUMENT_VERIFICATION_LEVEL_CONFIG,
            request,
            InstrumentVerificationLevelConfig,
        )

    def get_order_fee(
        self,
        request: GetOrderFeeRequest,
    ) -> OrderFee:
        """
        https://apidoc.notbank.exchange/#getorderfee
        """
        return self._get_data(
            Endpoints.GET_ORDER_FEE,
            request,
            OrderFee,
        )

    def ping(self) -> Pong:
        """
        https://apidoc.notbank.exchange/#ping
        """
        return self._get_data(Endpoints.PING, None, Pong, no_pascal_case=["msg"])

    def health_check(self) -> None:
        """
        https://apidoc.notbank.exchange/#healthcheck
        """
        return self._do_request(Endpoints.HEALTH_CHECK, None)

    def send_order_list(
        self,
        request: List[SendOrderRequest],
    ) -> None:
        """
        https://apidoc.notbank.exchange/#sendorderlist
        """
        internal_request = list(
            map(SendOrderRequestInternal.from_send_order_request, request)
        )
        return self._do_request(
            Endpoints.SEND_ORDER_LIST,
            internal_request
        )

    def send_cancel_list(self, request: List[CancelOrder]) -> None:
        """
        https://apidoc.notbank.exchange/#sendcancellist
        """
        return self._do_request(Endpoints.SEND_CANCEL_LIST, request)

    def send_cancel_replace_list(self, request: List[CancelReplaceOrderRequest]) -> None:
        """
        https://apidoc.notbank.exchange/#sendcancelreplacelist
        """
        return self._do_request(Endpoints.SEND_CANCEL_REPLACE_LIST, request)

    def modify_order(self, request: ModifyOrderRequest) -> None:
        """
        https://apidoc.notbank.exchange/#modifyorder
        """
        return self._do_request(Endpoints.MODIFY_ORDER, request)

    def cancel_all_orders(self, request: CancelAllOrdersRequest) -> None:
        """
        https://apidoc.notbank.exchange/#cancelallorders
        """
        return self._do_request(Endpoints.CANCEL_ALL_ORDERS, request)

    def get_order_status(self, request: GetOrderStatusRequest) -> Order:
        """
        https://apidoc.notbank.exchange/#getorderstatus
        """
        return self._get_data(Endpoints.GET_ORDER_STATUS, request, Order)

    def get_orders_history(self, request: GetOrdersHistoryRequest) -> List[Order]:
        """
        https://apidoc.notbank.exchange/#getordershistory
        """
        return self._get_ap_data_list(Endpoints.GET_ORDERS_HISTORY, request, Order)

    def get_trades_history(
        self,
        request: GetTradesHistoryRequest,
    ) -> List[TradeSummary]:
        """
        https://apidoc.notbank.exchange/#gettradeshistory
        """
        return self._get_ap_data_list(
            Endpoints.GET_TRADES_HISTORY,
            request,
            TradeSummary,
        )

    def get_order_history_by_order_id(
        self,
        request: GetOrderHistoryByOrderIdRequest,
    ) -> List[Order]:
        """
        https://apidoc.notbank.exchange/#getorderhistorybyorderid
        """
        return self._get_ap_data_list(
            Endpoints.GET_ORDER_HISTORY_BY_ORDER_ID,
            request,
            Order,
        )

    def get_ticker_history(
        self,
        request: GetTickerHistoryRequest,
    ) -> List[Ticker]:
        """
        https://apidoc.notbank.exchange/#gettickerhistory
        """
        def parse_ticker_list(data: Any) -> List[Ticker]:
            return [Ticker(
                end_date_time=ticker_raw[0],
                high=Decimal(str(ticker_raw[1])),
                low=Decimal(str(ticker_raw[2])),
                open=Decimal(str(ticker_raw[3])),
                close=Decimal(str(ticker_raw[4])),
                volume=Decimal(str(ticker_raw[5])),
                bid=Decimal(str(ticker_raw[6])),
                ask=Decimal(str(ticker_raw[7])),
                instrument_id=ticker_raw[8],
                begin_date_time=ticker_raw[9]
            ) for ticker_raw in data]

        return self._client_connection.request(
            Endpoints.GET_TICKER_HISTORY,
            EndpointCategory.AP,
            to_dict(request),
            parse_ticker_list
        )

    def get_last_trades(
        self,
        request: GetLastTradesRequest,
    ) -> List[PublicTrade]:
        """
        https://apidoc.notbank.exchange/#getlasttrades
        """
        def parse_trade_list(data: Any) -> List[PublicTrade]:
            return [PublicTrade(
                trade_id=elem[0],
                instrument_id=elem[1],
                quantity=elem[2],
                price=elem[3],
                order1=elem[4],
                order2=elem[5],
                trade_time=elem[6],
                direction=elem[7],
                taker_side=elem[8],
                block_trade=elem[9],
                order_client_id=elem[10]
            ) for elem in data]
        return self._client_connection.request(
            Endpoints.GET_LAST_TRADES,
            EndpointCategory.AP,
            to_dict(request),
            parse_trade_list,
        )

    def get_level1_summary(
        self,
    ) -> List[Level1TickerSummary]:
        """
        https://apidoc.notbank.exchange/#getlevel1summary
        """
        result: List[str] = self._client_connection.request(
            Endpoints.GET_LEVEL1_SUMMARY,
            EndpointCategory.AP,
            to_dict(GetLevel1SummaryRequest()),
            parse_response_fn=lambda data: data,
        )
        return list(map(lambda data: from_dict(Level1TickerSummary, json.loads(data, use_decimal=True)), result))

    def get_level1_summary_min(
        self,
        request: GetLevel1SummaryMinRequest,
    ) -> List[Level1TickerSummaryMin]:
        """
        https://apidoc.notbank.exchange/#getlevel1summarymin
        """
        return self._client_connection.request(
            Endpoints.GET_LEVEL1_SUMMARY_MIN,
            EndpointCategory.AP,
            to_dict(request),
            parse_response_fn=level1_ticker_summary_min_list_from_json_list_str,
        )

    def get_open_trade_reports(
        self,
        request: GetOpenTradeReportsRequest,
    ) -> List[Order]:
        """
        https://apidoc.notbank.exchange/#getopentradereports
        """
        return self._get_ap_data_list(
            Endpoints.GET_OPEN_TRADE_REPORTS,
            request,
            Order,
        )

    def get_orders(
        self,
        request: GetOrdersRequest,
    ) -> List[Order]:
        """
        https://apidoc.notbank.exchange/#getorders
        """
        return self._get_ap_data_list(
            Endpoints.GET_ORDERS,
            request,
            Order,
        )

    def get_order_history(
        self,
        request: GetOrderHistoryRequest,
    ) -> List[Order]:
        """
        https://apidoc.notbank.exchange/#getorderhistory
        """
        return self._get_ap_data_list(
            Endpoints.GET_ORDER_HISTORY,
            request,
            Order,
        )

    def send_order(
        self,
        request: SendOrderRequest,
    ) -> SendOrderResponse:
        """
        https://apidoc.notbank.exchange/#sendorder
        """
        internal_request = SendOrderRequestInternal.from_send_order_request(
            request
        )
        return self._get_data(
            endpoint=Endpoints.SEND_ORDER,
            request_data=internal_request,
            response_cls=SendOrderResponse,
            no_pascal_case=["status"]
        )

    def cancel_replace_order(
        self,
        request: CancelReplaceOrderRequest,
    ) -> CancelReplaceOrderResponse:
        """
        https://apidoc.notbank.exchange/#cancelreplaceorder
        """
        return self._get_data(
            Endpoints.CANCEL_REPLACE_ORDER,
            request,
            CancelReplaceOrderResponse,
        )

    def cancel_order(
        self,
        request: CancelOrderRequest,
    ) -> None:
        """
        https://apidoc.notbank.exchange/#cancelorder
        """
        return self._do_request(
            Endpoints.CANCEL_ORDER,
            request
        )

    def get_open_orders(self, request: GetOpenOrdersRequest) -> List[Order]:
        """
        https://apidoc.notbank.exchange/#getopenorders
        """
        return self._get_ap_data_list(Endpoints.GET_OPEN_ORDERS, request, Order)

    def get_account_trades(self, request: GetAccountTradesRequest) -> List[AccountTrade]:
        """
        https://apidoc.notbank.exchange/#getaccounttrades
        """
        return self._get_ap_data_list(Endpoints.GET_ACCOUNT_TRADES, request, AccountTrade)

    def get_summary(self) -> List[InstrumentSummary]:
        """
        https://apidoc.notbank.exchange/#summary
        """
        return self._get_ap_data_list(Endpoints.SUMMARY, None, InstrumentSummary)

    def get_ticker(self) -> Dict[str, TickerSummary]:
        """
        https://apidoc.notbank.exchange/#ticker
        """
        def response_fn(data): return {key: TickerSummary(
            **value) for key, value in data.items()}
        return self._client_connection.request(
            endpoint=Endpoints.TICKER,
            endpoint_category=EndpointCategory.AP,
            request_data=None,
            parse_response_fn=response_fn
        )

    def get_orderbook(self, request: OrderBookRequest) -> OrderBook:
        """
        https://apidoc.notbank.exchange/#orderbook
        """
        raw_orderbook = self._get_data(
            Endpoints.ORDER_BOOK, request, OrderBookRaw, no_pascal_case=["timestamp", "bids", "asks"])
        return order_book_from_raw(raw_orderbook)

    def get_order_book(self, request: OrderBookRequest) -> OrderBook:
        return self.get_orderbook(request)

    def get_trades(self, request: TradesRequest) -> List[TradeSummary]:
        """
        https://apidoc.notbank.exchange/#trades
        """
        return self._get_ap_data_list(Endpoints.TRADES, request, TradeSummary)

    def get_l2_snapshot(self, request: GetL2SnapshotRequest) -> List[Level2TickerSnapshot]:
        """
        https://apidoc.notbank.exchange/#getl2snapshot
        """
        def response_fn(data): return [Level2TickerSnapshot(
            md_update_id=item[0],
            number_of_unique_accounts=item[1],
            action_date_time=item[2],
            action_type=item[3],
            last_trade_price=item[4],
            number_of_orders=item[5],
            price=item[6],
            product_pair_code=item[7],
            quantity=item[8],
            side=item[9]
        ) for item in data]
        return self._client_connection.request(
            endpoint=Endpoints.GET_L2_SNAPSHOT,
            endpoint_category=EndpointCategory.AP,
            request_data=to_dict(request),
            parse_response_fn=response_fn
        )

    def get_level1(
        self,
        request: GetLevel1Request,
    ) -> Level1:
        """
        https://apidoc.notbank.exchange/#getlevel1
        """
        return self._get_data(
            Endpoints.GET_LEVEL1,
            request,
            Level1
        )

    def subscribe_level_1(
            self,
            request: SubscribeLevel1Request,
            snapshot_handler: Callable[[Level1], None],
            update_handler: Callable[[Level1], None]):
        """
        https://apidoc.notbank.exchange/#subscribelevel1
        """
        self._subscribe(
            WebSocketEndpoint.SUBSCRIBE_LEVEL1,
            request,
            [Callback(
                CallbackIdentifier.get(
                    WebSocketEndpoint.SUBSCRIBE_LEVEL1, request.instrument_id),
                build_subscription_handler(snapshot_handler, lambda json_str:from_json_str(Level1, json_str))),
             Callback(
                CallbackIdentifier.get(
                    WebSocketEndpoint.UPDATE_LEVEL_1, request.instrument_id),
                build_subscription_handler(update_handler, lambda json_str:from_json_str(Level1, json_str)))],
            lambda x: None)

    def subscribe_level_2(
        self,
        request: SubscribeLevel2Request,
            snapshot_handler: Callable[[List[Level2Feed]], None],
            update_handler: Callable[[List[Level2Feed]], None]):
        """
        https://apidoc.notbank.exchange/#subscribelevel2
        """
        instrument_id = None
        if request.instrument_id is not None:
            instrument_id = request.instrument_id
        if request.symbol is not None:
            instrument_id = self.get_instrument_id_by_symbol(request.symbol)
        self._subscribe(
            WebSocketEndpoint.SUBSCRIBE_LEVEL2,
            request,
            [Callback(
                CallbackIdentifier.get(
                    WebSocketEndpoint.SUBSCRIBE_LEVEL2, instrument_id),
                build_subscription_handler(snapshot_handler, lambda json_str:level_2_ticker_feed_list_from_json_str(json_str))),
             Callback(
                CallbackIdentifier.get(
                    WebSocketEndpoint.UPDATE_LEVEL_2, instrument_id),
                build_subscription_handler(update_handler, lambda json_str:level_2_ticker_feed_list_from_json_str(json_str)))],
            lambda x: None)

    def get_enums(self) -> List[EnumsResponse]:
        """
        https://apidoc.notbank.exchange/#getenums
        """
        return self._get_ap_data_list(
            Endpoints.GET_ENUMS,
            None,
            EnumsResponse
        )

    def get_user_accounts(self, request: GetUserAccountsRequest) -> List[int]:
        """
        https://apidoc.notbank.exchange/#getuseraccounts
        """
        def parse_response(data):
            return list(map(lambda account_id: int(account_id), data))

        return self._client_connection.request(
            Endpoints.GET_USER_ACCOUNTS,
            EndpointCategory.AP,
            to_dict(request),
            parse_response_fn=parse_response
        )

    def subscribe_ticker(
        self,
        request: SubscribeTickerRequest,
        snapshot_handler: Callable[[List[Ticker]], None],
        update_handler: Callable[[List[Ticker]], None]
    ):
        """
        https://apidoc.notbank.exchange/#subscribeticker
        """
        self._subscribe(
            WebSocketEndpoint.SUBSCRIBE_TICKER,
            request,
            [Callback(
                CallbackIdentifier.get(
                    WebSocketEndpoint.SUBSCRIBE_TICKER, request.instrument_id),
                build_subscription_handler(snapshot_handler, lambda json_str: ticker_list_from_json_str(json_str))),
             Callback(
                CallbackIdentifier.get(
                    WebSocketEndpoint.UPDATE_TICKER, request.instrument_id),
                build_subscription_handler(update_handler, lambda json_str: ticker_list_from_json_str(json_str)))],
            lambda x: None)

    def subscribe_trades(
            self,
            request: SubscribeTradesRequest,
            snapshot_handler: Callable[[List[PublicTrade]], None],
            update_handler: Callable[[List[PublicTrade]], None]):
        """
        https://apidoc.notbank.exchange/#subscribetrades
        """
        self._subscribe(
            WebSocketEndpoint.SUBSCRIBE_TRADES,
            request,
            [Callback(
                CallbackIdentifier.get(
                    WebSocketEndpoint.SUBSCRIBE_TRADES, request.instrument_id),
                build_subscription_handler(snapshot_handler, lambda json_str: public_trade_list_from_json_str(json_str))),
             Callback(
                CallbackIdentifier.get(
                    WebSocketEndpoint.UPDATE_TRADES, request.instrument_id),
                build_subscription_handler(update_handler, lambda json_str: public_trade_list_from_json_str(json_str)))],
            lambda x: None)

    def subscribe_order_state_events(
        self,
        request: SubscribeOrderStateEventsRequest,
        handler: Callable[[Order], None]
    ):
        """
        https://apidoc.notbank.exchange/#subscribeorderstateevents
        """
        if request.instrument_id is None:
            callback_id = CallbackIdentifier.get(
                WebSocketEndpoint.ACCOUNT_EVENT_ORDER_STATE, request.account_id)
        else:
            callback_id = CallbackIdentifier.get(
                WebSocketEndpoint.ACCOUNT_EVENT_ORDER_STATE, request.account_id, request.instrument_id)
        self._subscribe(
            WebSocketEndpoint.SUBSCRIBE_ORDER_STATE_EVENTS,
            request,
            [Callback(
                callback_id,
                build_subscription_handler(handler, lambda json_str: from_json_str(Order, json_str)))],
            lambda x: None)

    def subscribe_account_events(
            self,
            request: SubscribeAccountEventsRequest,
            *,
            withdraw_ticket_handler: Optional[Callable[[WithdrawTicket], None]] = None,
            transaction_handler: Optional[Callable[[AccountTransaction], None]] = None,
            trade_handler: Optional[Callable[[TradeSummary], None]] = None,
            order_handler: Optional[Callable[[Order], None]] = None,
            deposit_ticket_handler: Optional[Callable[[DepositTicket], None]] = None,
            account_handler: Optional[Callable[[AccountInfo], None]] = None,
            deposit_handler: Optional[Callable[[DepositEvent], None]] = None,
            cancel_order_reject_event_handler: Optional[Callable[[CancelOrderRejectEvent], None]] = None,
            balance_handler: Optional[Callable[[AccountPosition], None]] = None,
    ):
        """
        https://apidoc.notbank.exchange/#subscribeaccountevents
        """
        callbacks: List[Callback] = []
        if withdraw_ticket_handler is not None:
            callback = Callback(
                CallbackIdentifier.get(
                    WebSocketEndpoint.ACCOUNT_EVENT_WITHDRAW_TICKET_UPDATE, request.account_id),
                build_subscription_handler(withdraw_ticket_handler, lambda json_str: from_json_str(WithdrawTicket, json_str)))
            callbacks.append(callback)
        if transaction_handler is not None:
            callback = Callback(
                CallbackIdentifier.get(
                    WebSocketEndpoint.ACCOUNT_EVENT_TRANSACTION, request.account_id),
                build_subscription_handler(transaction_handler, lambda json_str: from_json_str(AccountTransaction, json_str)))
            callbacks.append(callback)
        if trade_handler is not None:
            callback = Callback(
                CallbackIdentifier.get(
                    WebSocketEndpoint.ACCOUNT_EVENT_ORDER_TRADE, request.account_id),
                build_subscription_handler(trade_handler, lambda json_str: from_json_str(TradeSummary, json_str)))
            callbacks.append(callback)
        if order_handler is not None:
            callback = Callback(
                CallbackIdentifier.get(
                    WebSocketEndpoint.ACCOUNT_EVENT_ORDER_STATE, request.account_id),
                build_subscription_handler(order_handler, lambda json_str: from_json_str(Order, json_str)))
            callbacks.append(callback)
        if deposit_ticket_handler is not None:
            callback = Callback(
                CallbackIdentifier.get(
                    WebSocketEndpoint.ACCOUNT_EVENT_DEPOSIT_TICKET_UPDATE, request.account_id),
                build_subscription_handler(deposit_ticket_handler, lambda json_str: from_json_str(DepositTicket, json_str)))
            callbacks.append(callback)
        if account_handler is not None:
            callback = Callback(
                CallbackIdentifier.get(
                    WebSocketEndpoint.ACCOUNT_EVENT_ACCOUNT_INFO_UPDATE, request.account_id),
                build_subscription_handler(account_handler, lambda json_str: from_json_str(AccountInfo, json_str)))
            callbacks.append(callback)
        if deposit_handler is not None:
            callback = Callback(
                CallbackIdentifier.get(
                    WebSocketEndpoint.ACCOUNT_EVENT_DEPOSIT, request.account_id),
                build_subscription_handler(deposit_handler, lambda json_str: from_json_str(DepositEvent, json_str)))
            callbacks.append(callback)
        if cancel_order_reject_event_handler is not None:
            callback = Callback(
                CallbackIdentifier.get(
                    WebSocketEndpoint.ACCOUNT_EVENT_CANCEL_ORDER_REJECT, request.account_id),
                build_subscription_handler(cancel_order_reject_event_handler, lambda json_str: from_json_str(CancelOrderRejectEvent, json_str)))
            callbacks.append(callback)
        if balance_handler is not None:
            callback = Callback(
                CallbackIdentifier.get(
                    WebSocketEndpoint.ACCOUNT_EVENT_ACCOUNT_POSITION, request.account_id),
                build_subscription_handler(balance_handler, lambda json_str: from_json_str(AccountPosition, json_str)))
            callbacks.append(callback)
        self._subscribe(
            WebSocketEndpoint.SUBSCRIBE_ACCOUNT_EVENTS,
            request,
            callbacks,
            lambda x: None)

    def unsubscribe_level_1(self, request: UnsubscribeLevel1Request) -> None:
        """
        https://apidoc.notbank.exchange/#unsubscribelevel1
        """
        self._unsubscribe(WebSocketEndpoint.UNSUBSCRIBE_LEVEL1, request, [
            CallbackIdentifier.get(
                WebSocketEndpoint.SUBSCRIBE_LEVEL1, request.instrument_id),
            CallbackIdentifier.get(
                WebSocketEndpoint.UPDATE_LEVEL_1, request.instrument_id)
        ],
            lambda x: None)

    def unsubscribe_level_2(self, request: UnsubscribeLevel2Request) -> None:
        """
        https://apidoc.notbank.exchange/#unsubscribelevel2
        """
        self._unsubscribe(WebSocketEndpoint.UNSUBSCRIBE_LEVEL2, request, [
            CallbackIdentifier.get(
                WebSocketEndpoint.SUBSCRIBE_LEVEL2, request.instrument_id),
            CallbackIdentifier.get(
                WebSocketEndpoint.UPDATE_LEVEL_2, request.instrument_id)
        ],
            lambda x: None)

    def unsubscribe_ticker(self, request: UnsubscribeTickerRequest) -> None:
        """
        https://apidoc.notbank.exchange/#unsubscribeticker
        """
        self._unsubscribe(WebSocketEndpoint.UNSUBSCRIBE_TICKER, request, [
            CallbackIdentifier.get(
                WebSocketEndpoint.SUBSCRIBE_TICKER, request.instrument_id),
            CallbackIdentifier.get(
                WebSocketEndpoint.UPDATE_TICKER, request.instrument_id)
        ],
            lambda x: None)

    def unsubscribe_trades(self, request: UnsubscribeTradesRequest) -> None:
        """
        https://apidoc.notbank.exchange/#unsubscribetrades
        """
        self._unsubscribe(WebSocketEndpoint.UNSUBSCRIBE_TRADES, request, [
            CallbackIdentifier.get(
                WebSocketEndpoint.SUBSCRIBE_TRADES, request.instrument_id),
            CallbackIdentifier.get(
                WebSocketEndpoint.UPDATE_TRADES, request.instrument_id),
        ],
            lambda x: None)

    def unsubscribe_order_state_events(self, request: UnsubscribeOrderStateEventsRequest):
        """
        https://apidoc.notbank.exchange/#unsubscribeorderstateevents
        """
        if request.instrument_id is None:
            callback_id = CallbackIdentifier.get(
                WebSocketEndpoint.ACCOUNT_EVENT_ORDER_STATE, request.account_id)
        else:
            callback_id = CallbackIdentifier.get(
                WebSocketEndpoint.ACCOUNT_EVENT_ORDER_STATE, request.account_id, request.instrument_id)
        self._unsubscribe(
            WebSocketEndpoint.UNSUBSCRIBE_TICKER,
            request,
            [callback_id],
            lambda x: None)

    def unsubscribe_account_events(self, request: UnsubscribeAccountEventsRequest):
        """
        https://apidoc.notbank.exchange/#unsubscribeaccountevents
        """
        callback_ids: List[str] = [
            CallbackIdentifier.get(
                WebSocketEndpoint.ACCOUNT_EVENT_WITHDRAW_TICKET_UPDATE, request.account_id),
            CallbackIdentifier.get(
                WebSocketEndpoint.ACCOUNT_EVENT_TRANSACTION, request.account_id),
            CallbackIdentifier.get(
                WebSocketEndpoint.ACCOUNT_EVENT_ORDER_TRADE, request.account_id),
            CallbackIdentifier.get(
                WebSocketEndpoint.ACCOUNT_EVENT_ORDER_STATE, request.account_id),
            CallbackIdentifier.get(
                WebSocketEndpoint.ACCOUNT_EVENT_DEPOSIT_TICKET_UPDATE, request.account_id),
            CallbackIdentifier.get(
                WebSocketEndpoint.ACCOUNT_EVENT_ACCOUNT_INFO_UPDATE, request.account_id),
            CallbackIdentifier.get(
                WebSocketEndpoint.ACCOUNT_EVENT_DEPOSIT, request.account_id),
            CallbackIdentifier.get(
                WebSocketEndpoint.ACCOUNT_EVENT_CANCEL_ORDER_REJECT, request.account_id),
        ]
        self._unsubscribe(
            WebSocketEndpoint.UNSUBSCRIBE_ACCOUNT_EVENTS,
            request,
            callback_ids,
            lambda x: None)

    def get_user_devices(self, request: GetUserDevicesRequest) -> List[UserDevice]:
        """
        https://apidoc.notbank.exchange/#getuserdevices
        """
        return self._get_ap_data_list(
            Endpoints.GET_USER_DEVICES,
            request,
            UserDevice
        )

    def get_user_info(self, request: GetUserInfoRequest) -> UserInfo:
        """
        https://apidoc.notbank.exchange/#getuserinfo
        """
        return self._get_data(
            Endpoints.GET_USER_INFO,
            request,
            UserInfo
        )

    def generate_trade_activity_report(
        self, request: GenerateTradeActivityReportRequest
    ) -> ActivityReport:
        """
        https://apidoc.notbank.exchange/#generatetradeactivityreport
        """
        return self._get_data(
            Endpoints.GENERATE_TRADE_ACTIVITY_REPORT,
            request,
            ActivityReport,
        )

    def generate_transaction_activity_report(
        self, request: GenerateTransactionActivityReportRequest
    ) -> ActivityReport:
        """
        https://apidoc.notbank.exchange/#generatetransactionactivityreport
        """
        return self._get_data(
            Endpoints.GENERATE_TRANSACTION_ACTIVITY_REPORT,
            request,
            ActivityReport
        )

    def generate_product_delta_activity_report(
        self, request: GenerateProductDeltaActivityReportRequest
    ) -> ActivityReport:
        """
        https://apidoc.notbank.exchange/#generateproductdeltaactivityreport
        """
        return self._get_data(
            Endpoints.GENERATE_PRODUCT_DELTA_ACTIVITY_REPORT,
            request,
            ActivityReport
        )

    def generate_pnl_activity_report(
        self, request: GeneratePnlActivityReportRequest
    ) -> ActivityReport:
        """
        https://apidoc.notbank.exchange/#generatepnlactivityreport
        """
        return self._get_data(
            Endpoints.GENERATE_PNL_ACTIVITY_REPORT,
            request,
            ActivityReport,
        )

    def schedule_trade_activity_report(
        self, request: ScheduleTradeActivityReportRequest
    ) -> ActivityReport:
        """
        https://apidoc.notbank.exchange/#healthcheck
        """
        return self._get_data(
            Endpoints.SCHEDULE_TRADE_ACTIVITY_REPORT,
            request,
            ActivityReport
        )

    def schedule_transaction_activity_report(
        self, request: ScheduleTransactionActivityReportRequest
    ) -> ActivityReport:
        """
        http://apidoc.notbank.exchange/#scheduletransactionactivityreport
        """
        return self._get_data(
            Endpoints.SCHEDULE_TRANSACTION_ACTIVITY_REPORT,
            request,
            ActivityReport
        )

    def schedule_product_delta_activity_report(
        self, request: ScheduleProductDeltaActivityReportRequest
    ) -> ActivityReport:
        """
        https://apidoc.notbank.exchange/#scheduleproductdeltaactivityreport
        """
        return self._get_data(
            Endpoints.SCHEDULE_PRODUCT_DELTA_ACTIVITY_REPORT,
            request,
            ActivityReport
        )

    def schedule_profit_and_loss_activity_report(
        self, request: ScheduleProfitAndLossActivityReportRequest
    ) -> ActivityReport:
        """
        https://apidoc.notbank.exchange/#scheduleprofitandlossactivityreport
        """
        return self._get_data(
            Endpoints.SCHEDULE_PROFIT_AND_LOSS_ACTIVITY_REPORT,
            request,
            ActivityReport
        )

    def cancel_user_report(self, request: CancelUserReportRequest) -> None:
        """
        https://apidoc.notbank.exchange/#canceluserreport
        """
        return self._do_request(
            Endpoints.CANCEL_USER_REPORT,
            request,
        )

    def get_user_report_writer_result_records(
        self, request: GetUserReportWriterResultRecordsRequest
    ) -> List[ReportWriterResultRecords]:
        """
        https://apidoc.notbank.exchange/#getuserreportwriterresultrecords
        """
        return self._get_ap_data_list(
            Endpoints.GET_USER_REPORT_WRITER_RESULT_RECORDS,
            request,
            ReportWriterResultRecords
        )

    def get_user_report_tickets(self, request: GetUserReportTicketsRequest) -> List[UserReportTicket]:
        """
        https://apidoc.notbank.exchange/#getuserreporttickets
        """
        return self._get_ap_data_list(
            Endpoints.GET_USER_REPORT_TICKETS,
            request,
            UserReportTicket
        )

    def remove_user_report_ticket(self, request: RemoveUserReportTicketRequest) -> None:
        """
        https://apidoc.notbank.exchange/#removeuserreportticket
        """
        payload = "{" + request.user_report_ticket_id + "}"
        return self._do_request(
            Endpoints.REMOVE_USER_REPORT_TICKET,
            payload
        )

    def get_user_report_tickets_by_status(self, request: GetUserReportTicketsByStatusRequest) -> List[UserReportTicket]:
        """
        https://apidoc.notbank.exchange/#getuserreportticketsbystatus
        """
        internal_request = convert_to_get_user_report_tickets_by_status_request_internal(
            request)
        return self._get_ap_data_list(
            Endpoints.GET_USER_REPORT_TICKETS_BY_STATUS,
            internal_request,
            UserReportTicket
        )

    def download_document(self, request: DownloadDocumentRequest) -> Document:
        """
        https://apidoc.notbank.exchange/#downloaddocument
        """
        return self._get_data(
            Endpoints.DOWNLOAD_DOCUMENT,
            request,
            Document
        )

    def download_document_slice(self, request: DownloadDocumentSliceRequest) -> DocumentSlice:
        """
        https://apidoc.notbank.exchange/#downloaddocumentslice
        """
        return self._get_data(
            Endpoints.DOWNLOAD_DOCUMENT_SLICE,
            request,
            DocumentSlice
        )

    def get_user_permissions(self, request: GetUserPermissionsRequest) -> List[str]:
        """
        https://apidoc.notbank.exchange/#getuserpermissions
        """
        return self._client_connection.request(
            Endpoints.GET_USER_PERMISSIONS,
            EndpointCategory.AP,
            to_dict(request),
            parse_response_fn=lambda x: x
        )

    # wallet

    def get_banks(self, request: GetBanksRequest) -> Banks:
        """
        https://apidoc.notbank.exchange/?http#getbanks
        """
        return self._client_connection.request(
            endpoint=Endpoints.BANKS,
            endpoint_category=EndpointCategory.NB_PAGE,
            request_data=to_nb_dict(request),
            parse_response_fn=parse_response_fn(Banks, from_pascal_case=False),
            request_type=RequestType.GET
        )

    def add_client_bank_account(self, request: AddClientBankAccountRequest) -> BankAccount:
        """
        https://apidoc.notbank.exchange/#addclientbankaccount
        """
        return self._get_nb_data(
            Endpoints.BANK_ACCOUNTS,
            request,
            BankAccount,
            endpoint_category=EndpointCategory.NB,
        )

    def get_client_bank_account(self, request: GetClientBankAccountRequest) -> BankAccount:
        """
        https://apidoc.notbank.exchange/#getclientbankaccount
        """
        return self._client_connection.request(
            endpoint=Endpoints.BANK_ACCOUNTS +
            "/" + str(request.bank_account_id),
            endpoint_category=EndpointCategory.NB,
            request_data=None,
            parse_response_fn=parse_response_fn(
                BankAccount, from_pascal_case=False),
            request_type=RequestType.GET

        )

    def get_client_bank_accounts(self, request: GetClientBankAccountsRequest) -> BankAccounts:
        """
        https://apidoc.notbank.exchange/#getclientbankaccounts
        """
        return self._client_connection.request(
            endpoint=Endpoints.BANK_ACCOUNTS,
            endpoint_category=EndpointCategory.NB_PAGE,
            request_data=to_nb_dict(request),
            parse_response_fn=parse_response_fn(
                BankAccounts, from_pascal_case=False),
            request_type=RequestType.GET
        )

    def delete_client_bank_account(self, request: DeleteClientBankAccountRequest) -> None:
        """
        https://apidoc.notbank.exchange/#deleteclientbankaccount
        """
        return self._client_connection.request(
            endpoint=Endpoints.BANK_ACCOUNTS +
            "/" + str(request.bank_account_id),
            endpoint_category=EndpointCategory.NB,
            request_data=None,
            parse_response_fn=lambda x: None,
            request_type=RequestType.DELETE
        )

    def get_networks_templates(self, request: GetNetworksTemplatesRequest) -> List[CurrencyNetworkTemplates]:
        """
        https://apidoc.notbank.exchange/?http#getnetworkstemplates
        """
        return self._client_connection.request(
            endpoint=Endpoints.GET_NETWORK_TEMPLATES,
            endpoint_category=EndpointCategory.NB,
            request_data=to_nb_dict(request),
            parse_response_fn=parse_response_list_fn(
                CurrencyNetworkTemplates, from_pascal_case=False),
            request_type=RequestType.GET
        )

    def get_deposit_addresses(self, request: DepositAddressRequest) -> List[str]:
        """
        https://apidoc.notbank.exchange/?http#getdepositaddresses
        """
        return self._client_connection.request(
            endpoint=Endpoints.GET_DEPOSIT_ADDRESSES,
            endpoint_category=EndpointCategory.NB,
            request_data=to_nb_dict(request),
            parse_response_fn=lambda x: x,
            request_type=RequestType.GET
        )

    def create_deposit_address(self, request: DepositAddressRequest) -> str:
        """
        https://apidoc.notbank.exchange/?http#createdepositaddress
        """
        return self._client_connection.request(
            endpoint=Endpoints.CREATE_DEPOSIT_ADDRESS,
            endpoint_category=EndpointCategory.NB,
            request_data=to_nb_dict(request),
            parse_response_fn=lambda x: x
        )

    def get_whitelisted_addresses(self, request: GetWhitelistedAddressesRequest) -> List[Address]:
        """
        https://apidoc.notbank.exchange/?http#getwhitelistedaddresses
        """
        return self._client_connection.request(
            endpoint=Endpoints.WHITELISTED_ADDRESSES,
            endpoint_category=EndpointCategory.NB,
            request_data=to_nb_dict(request),
            parse_response_fn=parse_response_list_fn(
                Address, from_pascal_case=False),
            request_type=RequestType.GET
        )

    def add_whitelisted_addresses(self, request: AddWhitelistedAddressRequest) -> UUID:
        """
        https://apidoc.notbank.exchange/?http#addwhitelistedaddress
        """
        return self._get_nb_data(
            Endpoints.WHITELISTED_ADDRESSES,
            request,
            IdResponse
        ).id

    def confirm_whitelisted_address(self, request: ConfirmWhiteListedAddressRequest) -> None:
        """
        https://apidoc.notbank.exchange/?http#confirmwhitelistedaddress
        """
        return self._client_connection.request(
            endpoint=Endpoints.WHITELISTED_ADDRESSES +
            "/" + str(request.whitelisted_address_id) + "/verification",
            endpoint_category=EndpointCategory.NB,
            request_data=to_nb_dict(ConfirmWhiteListedAddressRequestInternal(
                request.account_id, request.sms_code)),
            parse_response_fn=lambda x: None,
            request_type=RequestType.POST
        )

    def resend_verification_code_whitelisted_address(self, request: ResendVerificationCodeWhitelistedAddress) -> None:
        """
        https://apidoc.notbank.exchange/#resendverificationcodewhitelistedaddress
        """
        return self._client_connection.request(
            endpoint=Endpoints.WHITELISTED_ADDRESSES +
            "/" + str(request.whitelisted_address_id) + "/verification",
            endpoint_category=EndpointCategory.NB,
            request_data=to_nb_dict(
                ResendVerificationCodeWhitelistedAddressInternal(request.account_id)),
            parse_response_fn=lambda x: None,
            request_type=RequestType.GET
        )

    def delete_whitelisted_address(self, request: DeleteWhiteListedAddressRequest) -> None:
        """
        https://apidoc.notbank.exchange/?http#deletewhitelistedaddress
        """
        return self._client_connection.request(
            endpoint=Endpoints.WHITELISTED_ADDRESSES +
            "/" + str(request.whitelisted_address_id),
            endpoint_category=EndpointCategory.NB,
            request_data=to_nb_dict(DeleteWhiteListedAddressRequestInternal(
                account_id=request.account_id, otp=request.otp)),
            parse_response_fn=lambda x: None,
            request_type=RequestType.DELETE
        )

    def update_one_step_withdraw(self, request: UpdateOneStepWithdrawRequest) -> None:
        """
        https://apidoc.notbank.exchange/?http#updateonestepwithdraw
        """
        return self._client_connection.request(
            endpoint=Endpoints.UPDATE_ONE_STEP_WITHDRAW,
            endpoint_category=EndpointCategory.NB,
            request_data=to_nb_dict(request),
            parse_response_fn=lambda x: None,
            request_type=RequestType.POST
        )

    def create_crypto_withdraw(self, request: CreateCryptoWithdrawRequest) -> UUID:
        """
        https://apidoc.notbank.exchange/?http#createcriptowithdraw
        """
        return self._client_connection.request(
            endpoint=Endpoints.CREATE_CRIPTO_WITHDRAW,
            endpoint_category=EndpointCategory.NB,
            request_data=to_nb_dict(request),
            parse_response_fn=lambda x: UUID(x),
            request_type=RequestType.POST
        )

    def create_fiat_deposit(self, request: CreateFiatDepositRequest) -> Optional[str]:
        """
        https://apidoc.notbank.exchange/#createfiatdeposit
        """
        return self._client_connection.request(
            endpoint=Endpoints.FIAT_DEPOSIT,
            endpoint_category=EndpointCategory.NB,
            request_data=to_nb_dict(request),
            parse_response_fn=parse_response_fn(
                UrlResponse, from_pascal_case=False),
            request_type=RequestType.POST
        ).url

    def get_owners_fiat_withdraw(self, request: GetOwnersFiatWithdrawRequest) -> List[CbuOwner]:
        """
        https://apidoc.notbank.exchange/#getownersfiatwithdraw
        """
        return self._client_connection.request(
            endpoint=Endpoints.GET_OWNERS_FIAT_WITHDRAW,
            endpoint_category=EndpointCategory.NB,
            request_data=to_nb_dict(request),
            parse_response_fn=parse_response_list_fn(
                CbuOwner, from_pascal_case=False),
            request_type=RequestType.GET
        )

    def create_fiat_withdraw(self, request: CreateFiatWithdrawRequest) -> Optional[str]:
        """
        https://apidoc.notbank.exchange/#getownersfiatwithdraw
        """
        return self._client_connection.request(
            endpoint=Endpoints.FIAT_WITHDRAW,
            endpoint_category=EndpointCategory.NB,
            request_data=to_nb_dict(request),
            parse_response_fn=parse_response_fn(
                WithdrawalIdResponse, from_pascal_case=False),
            request_type=RequestType.POST
        ).withdrawal_id

    def confirm_fiat_withdraw(self, request: ConfirmFiatWithdrawRequest) -> None:
        """
        https://apidoc.notbank.exchange/#confirmfiatwithdraw
        """
        return self._client_connection.request(
            endpoint=Endpoints.FIAT_WITHDRAW+"/"+str(request.withdrawal_id),
            endpoint_category=EndpointCategory.NB,
            request_data=to_nb_dict(
                ConfirmFiatWithdrawRequestInternal(request.attempt_code)),
            parse_response_fn=lambda x: None,
            request_type=RequestType.POST
        )

    def transfer_funds(self, request: TransferFundsRequest) -> UUID:
        """
        https://apidoc.notbank.exchange/#transferfunds
        """
        return self._client_connection.request(
            endpoint=Endpoints.TRANSFER_FUNDS,
            endpoint_category=EndpointCategory.NB,
            request_data=to_nb_dict(request),
            parse_response_fn=lambda x: UUID(x),
            request_type=RequestType.POST
        )

    def get_transactions(self, request: GetTransactionsRequest) -> Transactions:
        """
        https://apidoc.notbank.exchange/#gettransactions
        """
        return self._client_connection.request(
            endpoint=Endpoints.GET_TRANSACTIONS,
            endpoint_category=EndpointCategory.NB_PAGE,
            request_data=to_nb_dict(request),
            parse_response_fn=parse_response_fn(
                Transactions, from_pascal_case=False),
            request_type=RequestType.GET
        )

    def get_quotes(self, request: GetQuotesRequest) -> List[Quote]:
        """
        https://apidoc.notbank.exchange/#getquotes
        """
        return self._client_connection.request(
            endpoint=Endpoints.QUOTES,
            endpoint_category=EndpointCategory.NB,
            request_data=to_nb_dict(request),
            parse_response_fn=parse_response_list_fn(
                Quote, from_pascal_case=False),
            request_type=RequestType.GET
        )

    def create_direct_quote(self, request: CreateDirectQuoteRequest) -> UUID:
        """
        https://apidoc.notbank.exchange/#createdirectquote
        """
        return self._client_connection.request(
            endpoint=Endpoints.QUOTES_DIRECT,
            endpoint_category=EndpointCategory.NB,
            request_data=to_nb_dict(request),
            parse_response_fn=parse_response_fn(
                UuidResponse, from_pascal_case=False),
            request_type=RequestType.POST
        ).id

    def create_inverse_quote(self, request: CreateInverseQuoteRequest) -> UUID:
        """
        https://apidoc.notbank.exchange/#createinversequote
        """
        return self._client_connection.request(
            endpoint=Endpoints.QUOTES_INVERSE,
            endpoint_category=EndpointCategory.NB,
            request_data=to_nb_dict(request),
            parse_response_fn=parse_response_fn(
                UuidResponse, from_pascal_case=False),
            request_type=RequestType.POST
        ).id

    def get_quote(self, request: GetQuoteRequest) -> Quote:
        """
        https://apidoc.notbank.exchange/#getquote
        """
        return self._client_connection.request(
            endpoint=Endpoints.QUOTES+"/"+str(request.quote_id),
            endpoint_category=EndpointCategory.NB,
            request_data=None,
            parse_response_fn=parse_response_fn(
                Quote, from_pascal_case=False),
            request_type=RequestType.GET
        )

    def execute_quote(self, request: ExecuteQuoteRequest) -> Quote:
        """
        https://apidoc.notbank.exchange/#executequote
        """
        return self._client_connection.request(
            endpoint=Endpoints.QUOTES+"/"+str(request.quote_id),
            endpoint_category=EndpointCategory.NB,
            request_data=None,
            parse_response_fn=parse_response_fn(
                Quote, from_pascal_case=False),
            request_type=RequestType.POST
        )



import unittest

from notbank_python_sdk.notbank_client import NotbankClient
from tests import test_helper


class TestInstrumentSummary(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_summary_success(self):
        response = self.client.get_summary()
        self.assertIsInstance(response, list)
        self.assertEqual(len(response), 2)
        self.assertEqual(response[0].trading_pairs, "BTC_USD")
        self.assertEqual(response[0].last_price, 29000)
        self.assertEqual(response[1].trading_pairs, "ETH_USD")
        self.assertEqual(response[1].last_price, 1970)


if __name__ == "__main__":
    unittest.main()



import unittest

from notbank_python_sdk.models.order_book import OrderBook
from notbank_python_sdk.requests_models.order_book import OrderBookRequest
from notbank_python_sdk.notbank_client import NotbankClient
from tests import test_helper
from tests.test_helper import new_websocket_client_connection


class TestOrderBook(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_websocket_client_connection()
        connection.connect()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    @classmethod
    def tearDownClass(cls):
        cls.client.close()

    def test_get_order_book(self):
        request = OrderBookRequest(
            market_pair="BTCUSD",
            level=-2,
            depth=10
        )
        orderbook = self.client.get_order_book(request)
        self.assertIsInstance(orderbook, OrderBook)


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from typing import Optional


@dataclass
class Address:
    id: str
    currency: str
    label: str
    network: str
    address: str
    verified: bool
    provider_id: int
    memo: Optional[str] = None



import json
from typing import Callable, Optional, Dict, Union
from notbank_python_sdk.core.endpoints import WebSocketEndpoint

from notbank_python_sdk.websocket.message_frame import MessageFrame


class CallbackIdentifier:
    _mapping: Dict[WebSocketEndpoint, Callable[[str, str], str]] = {
        WebSocketEndpoint.SUBSCRIBE_LEVEL_1: lambda event_name, payload: CallbackIdentifier._get_level1_ticker_name(event_name, payload),
        WebSocketEndpoint.UPDATE_LEVEL_1: lambda event_name, payload: CallbackIdentifier._get_level1_ticker_name(event_name, payload),
        WebSocketEndpoint.SUBSCRIBE_LEVEL_2: lambda event_name, payload: CallbackIdentifier._get_level2_ticker_name(event_name, payload),
        WebSocketEndpoint.UPDATE_LEVEL_2: lambda event_name, payload: CallbackIdentifier._get_level2_ticker_name(event_name, payload),
        WebSocketEndpoint.SUBSCRIBE_TICKER: lambda event_name, payload: CallbackIdentifier._get_ticker_name(event_name, payload),
        WebSocketEndpoint.UPDATE_TICKER: lambda event_name, payload: CallbackIdentifier._get_ticker_name(event_name, payload),
        WebSocketEndpoint.SUBSCRIBE_TRADES: lambda event_name, payload: CallbackIdentifier._get_socket_trade_name(event_name, payload),
        WebSocketEndpoint.UPDATE_TRADES: lambda event_name, payload: CallbackIdentifier._get_socket_trade_name(event_name, payload),
        WebSocketEndpoint.SUBSCRIBE_ORDER_STATE_EVENTS: lambda event_name, payload: CallbackIdentifier._get_order_event_name(event_name, payload),
        WebSocketEndpoint.ACCOUNT_EVENT_TRANSACTION: lambda event_name, payload: CallbackIdentifier._get_account_event_name(event_name, payload),
        WebSocketEndpoint.ACCOUNT_EVENT_WITHDRAW_TICKET_UPDATE: lambda event_name, payload: CallbackIdentifier._get_account_event_name(event_name, payload),
        WebSocketEndpoint.ACCOUNT_EVENT_ACCOUNT_POSITION: lambda event_name, payload: CallbackIdentifier._get_account_event_name(event_name, payload),
        WebSocketEndpoint.ACCOUNT_EVENT_ORDER_TRADE: lambda event_name, payload: CallbackIdentifier._get_account_event_name(event_name, payload),
        WebSocketEndpoint.ACCOUNT_EVENT_ORDER_STATE: lambda event_name, payload: CallbackIdentifier._get_account_event_name(event_name, payload),
        WebSocketEndpoint.ACCOUNT_EVENT_DEPOSIT_TICKET_UPDATE: lambda event_name, payload: CallbackIdentifier._get_account_event_name(event_name, payload),
        WebSocketEndpoint.ACCOUNT_EVENT_ACCOUNT_INFO_UPDATE: lambda event_name, payload: CallbackIdentifier._get_account_event_name(event_name, payload),
        WebSocketEndpoint.ACCOUNT_EVENT_CANCEL_ORDER_REJECT: lambda event_name, payload: CallbackIdentifier._get_account_event_name(event_name, payload),
        WebSocketEndpoint.ACCOUNT_EVENT_DEPOSIT: lambda event_name, payload: CallbackIdentifier._get_account_event_name(event_name, payload),
    }

    @staticmethod
    def get(event_name: str, first_identifier: Optional[int] = None, second_identifier: Optional[int] = None) -> str:
        return event_name + CallbackIdentifier._get_id_chunk(first_identifier) + CallbackIdentifier._get_id_chunk(second_identifier)

    @staticmethod
    def get_from_message_frame(message: MessageFrame) -> Optional[str]:
        try:
            websocket_endpoint = WebSocketEndpoint(message.n)
        except ValueError as e:
            return None
        mapping_function = CallbackIdentifier._mapping.get(websocket_endpoint)
        return mapping_function(message.n, message.o) if mapping_function else message.n

    @staticmethod
    def _get_ticker_name(event_name: str, payload_str: str) -> str:
        instrument_id = CallbackIdentifier._get_value_from_list(
            payload_str, 8)
        return event_name + CallbackIdentifier._get_id_chunk(instrument_id)

    @staticmethod
    def _get_level1_ticker_name(event_name: str, payload_str: str) -> str:
        instrument_id = CallbackIdentifier._get_instrumented_id(
            payload_str)
        return event_name + CallbackIdentifier._get_id_chunk(instrument_id)

    @staticmethod
    def _get_level2_ticker_name(event_name: str, payload_str: str) -> str:
        data = json.loads(payload_str)
        if not data or len(data) < 1 or len(data[0]) < 8:
            return event_name
        level2_ticker = data[0]
        instrument_id = level2_ticker[7]
        return event_name + CallbackIdentifier._get_id_chunk(instrument_id)

    @staticmethod
    def _get_socket_trade_name(event_name: str, payload_str: str) -> str:
        instrument_id = CallbackIdentifier._get_value_from_list(
            payload_str, 1)
        return event_name + CallbackIdentifier._get_id_chunk(instrument_id)

    @staticmethod
    def _get_account_event_name(event_name: str, payload_str: str) -> str:
        account_id = CallbackIdentifier._get_account_id(
            payload_str) or CallbackIdentifier._get_account_id_from_account(payload_str)
        return event_name + CallbackIdentifier._get_id_chunk(account_id)

    @staticmethod
    def _get_order_event_name(event_name: str, payload_str: str) -> str:
        instrument_id = CallbackIdentifier._get_instrumented_id_from_instrument(
            payload_str)
        return event_name + CallbackIdentifier._get_id_chunk(instrument_id)

    @staticmethod
    def _get_id_chunk(identifier: Optional[Union[int, str]]) -> str:
        return f"_{identifier}" if identifier is not None else ""

    @staticmethod
    def _get_value_from_list(payload_str: str, index: int) -> Optional[str]:
        data = json.loads(payload_str)
        return data[0][index] if data else None

    @staticmethod
    def _get_instrumented_id(payload_str: str) -> str:
        data = json.loads(payload_str)
        return data.get("InstrumentId")

    @staticmethod
    def _get_instrumented_id_from_instrument(payload_str: str) -> Optional[str]:
        data = json.loads(payload_str)
        return data.get("Instrument")

    @staticmethod
    def _get_account_id(payload_str: str) -> Optional[str]:
        data = json.loads(payload_str)
        return data.get("AccountId")

    @staticmethod
    def _get_account_id_from_account(payload_str: str) -> Optional[str]:
        data = json.loads(payload_str)
        return data.get("Account")



from dataclasses import dataclass
from decimal import Decimal
import simplejson as json
from typing import Any, List
from notbank_python_sdk.core.either import Either

from notbank_python_sdk.error import ErrorCode, NotbankException


@dataclass
class PublicTrade:
    trade_id: int
    instrument_id: int
    quantity: Decimal
    price: Decimal
    order1: int
    order2: int
    trade_time: int
    direction: int
    taker_side: int
    block_trade: bool
    order_client_id: int


def public_trade_from_list(list_str: List[Any]) -> PublicTrade:
    return PublicTrade(
        trade_id=list_str[0],
        instrument_id=list_str[1],
        quantity=list_str[2],
        price=list_str[3],
        order1=list_str[4],
        order2=list_str[5],
        trade_time=list_str[6],
        direction=list_str[7],
        taker_side=list_str[8],
        block_trade=list_str[9] == 1,
        order_client_id=list_str[10]
    )


def public_trade_list_from_json_str(json_str: str) -> Either[NotbankException, List[PublicTrade]]:
    try:
        json_list = json.loads(json_str, use_decimal=True)
        return Either.right([public_trade_from_list(item) for item in json_list])
    except Exception as e:
        return Either.left(NotbankException(ErrorCode.CONFIGURATION_ERROR, str(e)))



from dataclasses import dataclass
from typing import Optional, List


@dataclass
class GetUserReportTicketsByStatusRequestItem:
    request_status: str


@dataclass
class GetUserReportTicketsByStatusRequestInternal:
    request_statuses: Optional[List[GetUserReportTicketsByStatusRequestItem]] = None


@dataclass
class GetUserReportTicketsByStatusRequest:
    request_statuses: Optional[List[str]] = None


def convert_to_get_user_report_tickets_by_status_request_internal(get_user_report_tickets_by_status_request: GetUserReportTicketsByStatusRequest) -> GetUserReportTicketsByStatusRequestInternal:
    if get_user_report_tickets_by_status_request.request_statuses is None:
        return GetUserReportTicketsByStatusRequestInternal()
    statuses = [GetUserReportTicketsByStatusRequestItem(
        item) for item in get_user_report_tickets_by_status_request.request_statuses]
    return GetUserReportTicketsByStatusRequestInternal(statuses)



import unittest

from notbank_python_sdk.models.send_order import SendOrderStatus
from notbank_python_sdk.requests_models.get_account_positions_request import GetAccountPositionsRequest
from notbank_python_sdk.requests_models.order_book import OrderBookRequest
from notbank_python_sdk.requests_models.send_order import OrderType, SendOrderRequest, Side, TimeInForce
from tests import test_helper

from notbank_python_sdk.requests_models import *


class TestSendOrder(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.credentials = test_helper.load_credentials()

    def test_send_order_success(self):
        from decimal import Decimal
        import random

        from notbank_python_sdk.notbank_client import NotbankClient
        from notbank_python_sdk.client_connection_factory import new_rest_client_connection

        account_id: int = 13  # must be user account id

        # instantiate client
        test_url = "stgapi.notbank.exchange"
        rest_connection = new_rest_client_connection(test_url)
        client = NotbankClient(rest_connection)

        # authentication (same for rest client or websocket client)
        authenticate = client.authenticate(
            AuthenticateRequest(
                api_public_key=self.credentials.public_key,
                api_secret_key=self.credentials.secret_key,
                user_id=self.credentials.user_id,
            )
        )
        if not authenticate.authenticated:
            raise Exception("client not authenticated")

        # get USDT user balance (also known as position)
        positions = client.get_account_positions(
            GetAccountPositionsRequest(account_id))
        usdt_balance = None
        product = "USDT"
        market_pair = "BTCUSDT"
        for position in positions:
            if position.product_symbol == product:
                usdt_balance = position
        if usdt_balance is None:
            raise Exception("user has no balance")

        # define order_amount (between all usdt_balance and half usdt_balance)
        total_balance = usdt_balance.amount
        quantity_to_spend = total_balance - \
            Decimal(random.random()) * (total_balance/2)

        # define order_price (around market top)
        orderbook = client.get_order_book(
            OrderBookRequest(market_pair, level=2, depth=5))
        top_orderbook = orderbook.bids[0]
        delta = Decimal(random.randrange(10, 100))/1000
        order_price = top_orderbook.price + delta

        order_quantity = quantity_to_spend / order_price

        # send order
        instrument = client.get_instrument_by_symbol(market_pair)
        request = SendOrderRequest(
            instrument=instrument,
            account_id=account_id,
            time_in_force=TimeInForce.GTC,
            side=Side.Buy,
            quantity=order_quantity,
            limit_price=order_price,
            order_type=OrderType.Limit,
        )
        response = client.send_order(request)
        # handle order result
        if response.status == SendOrderStatus.REJECTED:
            # order was rejected
            raise Exception("rejected order")
        else:
            # order was accepted
            order_id = response.order_id

        client.close()


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from decimal import Decimal
from typing import Optional


@dataclass
class Order:
    side: str
    order_id: int
    price: Decimal
    quantity: Decimal
    display_quantity: Decimal
    instrument: int
    account: int
    account_name: str
    order_type: str
    client_order_id: int
    order_state: str
    receive_time: int
    receive_time_ticks: int
    last_updated_time: int
    last_updated_time_ticks: int
    orig_quantity: Decimal
    quantity_executed: Decimal
    gross_value_executed: Decimal
    executable_value: Decimal
    avg_price: Decimal
    counter_party_id: int
    change_reason: str
    orig_order_id: int
    orig_cl_ord_id: int
    entered_by: int
    user_name: str
    is_quote: bool
    inside_ask: Decimal
    inside_ask_size: Decimal
    inside_bid: Decimal
    inside_bid_size: Decimal
    last_trade_price: Decimal
    reject_reason: str
    is_locked_in: bool
    cancel_reason: str
    order_flag: str
    use_margin: bool
    stop_price: Decimal
    peg_price_type: str
    peg_offset: Decimal
    peg_limit_offset: Decimal
    oms_id: int
    ip_address: Optional[str] = None
    ipv6a: Optional[int] = None
    ipv6b: Optional[int] = None
    client_order_id_uuid: Optional[str] = None



import unittest
from unittest.mock import MagicMock
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.core.endpoints import WebSocketEndpoint
from notbank_python_sdk.models.web_authenticate_user_websocket import WebAuthenticateUserResponse
from notbank_python_sdk.requests_models.web_authenticate_user_request import WebAuthenticateUserRequest
from tests import test_helper


class TestWebAuthenticateUserIntegration(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_websocket_client_connection()
        cls.credentials = test_helper.load_credentials()
        connection.connect()
        cls.client = NotbankClient(connection)

    def test_initialization(self):
        response = self.client.web_authenticate_user(WebAuthenticateUserRequest(
            session_token="existingsessiontoken",

        ))


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from decimal import Decimal


@dataclass
class AccountFee:
    fee_id: int
    oms_id: int
    fee_tier: int
    account_id: int
    fee_amt: Decimal
    fee_calc_type: str
    fee_type: str
    ladder_threshold: Decimal
    ladder_seconds: int
    is_active: bool
    instrument_id: int
    order_type: str
    pegged_product_id: int



from typing import Callable, Dict, Generic, Optional, TypeVar

T = TypeVar('T')


class SequencedCallbacks(Generic[T]):
    _sequence_number: int
    _callbacks: Dict[int, Callable[[T], None]]

    def __init__(self, sequence_number: int, callbacks: Dict[int, Callable[[T], None]]):
        self._sequence_number = sequence_number
        self._callbacks = callbacks

    @staticmethod
    def create() -> "SequencedCallbacks":
        return SequencedCallbacks(sequence_number=2, callbacks={})

    def pop(self, sequence_number: int) -> Optional[Callable[[T], None]]:
        return self._callbacks.pop(sequence_number, None)

    def put(self, callback: Callable[[T], None]) -> int:
        sequence_number = self._get_next_sequence_number()
        self._callbacks[sequence_number] = callback
        return sequence_number

    def _get_next_sequence_number(self) -> int:
        sequence_number = self._sequence_number
        self._sequence_number += 2
        return sequence_number

    def close(self) -> None:
        self._callbacks.clear()



import unittest

from notbank_python_sdk.requests_models.cancel_all_orders import CancelAllOrdersRequest
from notbank_python_sdk.notbank_client import NotbankClient
from tests import test_helper


class TestCancelAllOrders(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_cancel_all_orders_success(self):
        request = CancelAllOrdersRequest(
            account_id=self.credentials.account_id,
            instrument_id=1,
        )
        try:
            self.client.cancel_all_orders(request)
        except Exception as e:
            self.fail(f"Cancel all orders failed with exception: {e}")

    def test_cancel_all_orders_invalid_omsid(self):
        request = CancelAllOrdersRequest(
            account_id=self.credentials.account_id+123,
            instrument_id=66,
        )
        try:
            self.client.cancel_all_orders(request)
        except Exception as e:
            self.assertIsInstance(e, Exception)


if __name__ == "__main__":
    unittest.main()



from decimal import Decimal
import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_trades_history import GetTradesHistoryRequest
from tests import test_helper


class TestGetTradesHistory(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_trades_history_success(self):
        """Prueba exitosa: oms_id y account_id válidos."""
        request = GetTradesHistoryRequest(account_id=7)
        response = self.client.get_trades_history(request)

        # Verificar que devuelve correctamente 1 trade
        self.assertEqual(len(response), 1)
        self.assertEqual(response[0].trade_id, 964)
        self.assertEqual(response[0].price, Decimal(6000.0))

    def test_get_trades_history_not_found(self):
        """Prueba: account_id inválido, no se encuentra el registro."""
        request = GetTradesHistoryRequest(account_id=999)
        response = self.client.get_trades_history(request)

        # Verificar que no se devuelven trades
        self.assertEqual(len(response), 0)


# Punto de entrada para ejecutar pruebas
if __name__ == "__main__":
    unittest.main()



from typing import Any, Callable, Optional

from notbank_python_sdk.client_connection import ClientConnection
from notbank_python_sdk.rest.rest_client_connection import RestClientConnection
from notbank_python_sdk.websocket.connection_configuration import ConnectionConfiguration
from notbank_python_sdk.websocket.websocket_connection import WebsocketConnection
from notbank_python_sdk.websocket.websocket_restarter.restarter import Restarter
from notbank_python_sdk.websocket.websocket_restarter.restarting_websocket_connection import RestartingWebsocketConnection


def _get_not_implemented(method_name: str, client_name: str) -> Callable[..., None]:
    def _not_implemented(*args: Any, **kwargs: Any) -> None:
        raise NotImplementedError(
            f"{method_name} is not implemented in {client_name}.")
    return _not_implemented


API_NOTBANK_EXCHANGE = "api.notbank.exchange"


def new_websocket_client_connection(
    url: str = API_NOTBANK_EXCHANGE,
    on_open: Callable[[], None] = lambda: None,
    on_close: Callable[[Any, str], None] = lambda code, message: None,
    on_failure: Callable[[Exception], None] = lambda e: None,
    peek_message_in: Callable[[str], None] = lambda x: None,
    peek_message_out: Callable[[str], None] = lambda x: None,
    request_timeout: Optional[float] = None,
) -> ClientConnection:
    client_restarter = WebsocketConnection.create(
        url, on_open, on_close, on_failure, peek_message_in, peek_message_out, request_timeout)
    return ClientConnection(
        post_request=client_restarter.request,
        get_request=client_restarter.request,
        delete_request=_get_not_implemented(
            "delete request", "WebsocketClientConnection"),
        subscribe=client_restarter.subscribe,
        unsubscribe=client_restarter.unsubscribe,
        authenticate_user=lambda request_message: client_restarter.authenticate_user(
            request_message),
        connect=client_restarter.connect,
        close=client_restarter.close,
    )


def new_restarting_websocket_client_connection(
    url: str = API_NOTBANK_EXCHANGE,
    on_open: Callable[[], None] = lambda: None,
    on_close: Callable[[Any, str], None] = lambda code, message: None,
    on_failure: Callable[[Exception], None] = lambda e: None,
    peek_message_in: Callable[[str], None] = lambda x: None,
    peek_message_out: Callable[[str], None] = lambda x: None,
    request_timeout: Optional[float] = None,
) -> ClientConnection:
    restarter = Restarter.create(
        ConnectionConfiguration(
            url, on_open, on_close, on_failure, peek_message_in, peek_message_out, request_timeout)
    )
    restarting_websocket_connection = RestartingWebsocketConnection(restarter)
    return ClientConnection(
        post_request=restarting_websocket_connection.request,
        get_request=restarting_websocket_connection.request,
        delete_request=_get_not_implemented(
            "delete request", "WebsocketClientConnection"),
        subscribe=restarting_websocket_connection.subscribe,
        unsubscribe=restarting_websocket_connection.unsubscribe,
        authenticate_user=restarting_websocket_connection.authenticate_user,
        connect=restarting_websocket_connection.connect,
        close=restarting_websocket_connection.close,
    )


def new_rest_client_connection(
    url: str = API_NOTBANK_EXCHANGE,
    peek_message_in: Callable[[str], None] = lambda a: None,
    peek_message_out: Callable[[str, Any, Any, str,
                                dict], None] = lambda a, b, c, d, e: None,
) -> ClientConnection:
    rest_client_connection = RestClientConnection(
        url, peek_message_in=peek_message_in, peek_message_out=peek_message_out)
    return ClientConnection(
        post_request=rest_client_connection.post,
        get_request=rest_client_connection.get,
        delete_request=rest_client_connection.delete,
        subscribe=_get_not_implemented("subscription", "RestClientConnection"),
        unsubscribe=_get_not_implemented(
            "unsubscription", "RestClientConnection"),
        authenticate_user=rest_client_connection.authenticate_user,
        connect=_get_not_implemented("connect", "RestClientConnection"),
        close=rest_client_connection.close,
    )



from dataclasses import dataclass


@dataclass
class ReportWriterResultRecords:
    requesting_user: int
    urt_ticket_id: str
    descriptor_id: str
    result_status: str
    report_execution_start_time: str
    report_execution_complete_time: str
    report_output_file_pathname: str
    report_descriptive_header: str





import unittest
from notbank_python_sdk.notbank_client import NotbankClient
from notbank_python_sdk.requests_models.add_whitelisted_address_request import AddWhitelistedAddressRequest
from notbank_python_sdk.requests_models.get_whitelisted_addresses_request import GetWhitelistedAddressesRequest
from tests import test_helper


class TestAddWhiteListedAddresses(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection(
            test_helper.print_message_in, test_helper.print_message_out)
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_add_whitelisted_addresses(self):
        response = self.client.add_whitelisted_addresses(
            AddWhitelistedAddressRequest(
                self.credentials.account_id,
                currency="BTC",
                network="BTC_TEST",
                address="tb1qrekfjtdrffe7r62fr4hspgka8qmzq0t3wu9z87",
                label="a label",
                otp="047973"))
        self.assertIsNotNone(response)


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from typing import Optional


@dataclass
class GetUserInfoRequest:
    user_id: Optional[int] = None
    email: Optional[str] = None
    username: Optional[str] = None
    affiliate_id: Optional[int] = None



import unittest
from notbank_python_sdk.requests_models.deposit_address_request import DepositAddressRequest
from notbank_python_sdk.requests_models.get_account_info_request import GetAccountInfoRequest

from tests import test_helper

from notbank_python_sdk.notbank_client import NotbankClient
from notbank_python_sdk.requests_models.get_banks_request import GetBanksRequest


class TestGetDepositAddresses(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection(
            test_helper.print_message_in, test_helper.print_message_out)
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_deposit_addresses(self):
        response = self.client.get_deposit_addresses(
            DepositAddressRequest(self.credentials.account_id, "BTC", "BTC_TEST"))
        self.assertIsNotNone(response)
    

if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass


@dataclass
class UserDevice:
    hash_code: int
    location: str
    device_name: str
    ip_address: str
    user_id: int
    is_trusted: bool
    expiration_time: int



from dataclasses import dataclass
from decimal import Decimal
from typing import Optional, Union

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


@dataclass
class CancelReplaceOrderRequest(WithOMSId):
    order_id_to_replace: int
    client_order_id: Optional[int] = None
    order_type: Optional[Union[int, str]] = None
    side: Optional[Union[int, str]] = None
    account_id: Optional[int] = None
    instrument_id: Optional[int] = None
    use_display_quantity: Optional[bool] = False
    display_quantity: Optional[Decimal] = None
    limit_price: Optional[Decimal] = None
    stop_price: Optional[Decimal] = None
    reference_price: Optional[Decimal] = None
    peg_price_type: Optional[Union[int, str]] = None
    time_in_force: Optional[Union[int, str]] = None
    order_id_oco: Optional[int] = None
    quantity: Optional[Decimal] = None



from decimal import Decimal
import unittest
from uuid import UUID
from notbank_python_sdk.requests_models.create_fiat_deposit_request import CreateFiatDepositRequest
from notbank_python_sdk.requests_models.get_account_info_request import GetAccountInfoRequest
from notbank_python_sdk.requests_models.transfer_funds_request import TransferFundsRequest

from tests import test_helper

from notbank_python_sdk.notbank_client import NotbankClient
from notbank_python_sdk.requests_models.get_banks_request import GetBanksRequest


class TestTransferFunds(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection(
            test_helper.print_message_in, test_helper.print_message_out)
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_transfer_funds(self):
        response = self.client.transfer_funds(TransferFundsRequest(
            sender_account_id=235,
            receiver_account_id=13,
            currency_name="BTC",
            amount=Decimal("0.01"),
            otp="470888",
            notes="a test transfer",
            
        ))
        self.assertIsNotNone(response)


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from typing import List, Optional

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


@dataclass
class ScheduleTradeActivityReportRequest(WithOMSId):
    begin_time: str
    account_id_list: List[int]
    frequency: Optional[str] = None



from decimal import Decimal


def truncate_dec(value: Decimal, decimals: Decimal) -> Decimal:
    exp = Decimal(1/decimals)
    return Decimal(int(value * exp)) / Decimal(exp)


def dec_to_str_stripped(val: Decimal) -> str:
    if val == Decimal("0"):
        return "0"
    dec_str = "{0:f}".format(val)
    return dec_str.rstrip("0").rstrip(".") if "." in dec_str else dec_str




import unittest
from notbank_python_sdk.requests_models.add_client_bank_account_request import AddClientBankAccountRequest
from notbank_python_sdk.requests_models.get_account_info_request import GetAccountInfoRequest
from notbank_python_sdk.requests_models.get_network_templates_request import GetNetworksTemplatesRequest

from tests import test_helper

from notbank_python_sdk.notbank_client import NotbankClient
from notbank_python_sdk.requests_models.get_banks_request import GetBanksRequest


class TestGetNetworksTemplates(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection(
            peek_message_out=test_helper.print_message_out, peek_message_in=test_helper.print_message_in)
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_networks_templates(self):
        response = self.client.get_networks_templates(
            GetNetworksTemplatesRequest(currency="BTC"))
        self.assertIsNotNone(response)


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from decimal import Decimal
from enum import Enum


class TransactionType(str, Enum):
    fee = "Fee"
    trade = "Trade"
    other = "Other"
    reverse = "Reverse"
    hold = "Hold"
    rebate = "Rebate"
    margin_acquisition = "MarginAcquisition"
    margin_relinquish_by_trade = "MarginRelinquishByTrade"
    margin_interest_transfer = "MarginInterestTransfer"
    margin_operator_transfer_to_margin_account = "MarginOperatorTransferToMarginAccount"
    margin_operator_transfer_to_asset_account = "MarginOperatorTransferToAssetAccount"
    margin_user_transfer = "MarginUserTransfer"
    margin_relinquish_by_transfer = "MarginRelinquishByTransfer"
    margin_relinquish_by_reverse_trade = "MarginRelinquishByReverseTrade"
    distribution = "Distribution"
    payment = "Payment"
    operator_lend = "OperatorLend"
    operator_received = "OperatorReceived"
    rebalance = "Rebalance"
    commission = "Commission"
    air_drop = "AirDrop"


class TransactionReferenceType(str, Enum):
    trade = "Trade"
    deposit = "Deposit"
    withdraw = "Withdraw"
    transfer = "Transfer"
    order_hold = "OrderHold"
    withdraw_hold = "WithdrawHold"
    deposit_hold = "DepositHold"
    margin_hold = "MarginHold"
    manual_hold = "ManualHold"
    manual_entry = "ManualEntry"
    margin_acquisition = "MarginAcquisition"
    margin_relinquish = "MarginRelinquish"
    margin_interest_netting = "MarginInterestNetting"
    margin_operator_transfer_to_margin_account = "MarginOperatorTransferToMarginAccount"
    margin_operator_transfer_to_asset_account = "MarginOperatorTransferToAssetAccount"
    margin_user_transfer = "MarginUserTransfer"
    margin_position_reverse_trade = "MarginPositionReverseTrade"
    affiliate_rebate = "AffiliateRebate"
    distribution_entry = "DistributionEntry"
    transfer_hold = "TransferHold"
    air_drop = "AirDrop"


@dataclass
class AccountTransaction:
    transaction_id: int
    reference_id: int
    oms_id: int
    account_id: int
    cr: Decimal  # Crédito
    dr: Decimal  # Débito
    counterparty: int
    transaction_type: TransactionType
    reference_type: TransactionReferenceType
    product_id: int
    time_stamp: int
    balance: Decimal = Decimal(0)



import unittest
from uuid import UUID
from notbank_python_sdk.requests_models.delete_client_bank_account_request import DeleteClientBankAccountRequest
from notbank_python_sdk.requests_models.get_account_info_request import GetAccountInfoRequest

from tests import test_helper

from notbank_python_sdk.notbank_client import NotbankClient
from notbank_python_sdk.requests_models.get_banks_request import GetBanksRequest


class TestDeleteClientBankAccounts(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_delete_client_bank_account(self):
        response = self.client.delete_client_bank_account(
            DeleteClientBankAccountRequest(UUID('b49940d1-abf8-452a-a8a4-ece70bf53412')))
        self.assertIsNone(response)


if __name__ == "__main__":
    unittest.main()



from enum import Enum


class Endpoints(str, Enum):

    # account
    GET_ACCOUNT_TRANSACTIONS = "GetAccountTransactions"
    GET_ACCOUNT_POSITIONS = "GetAccountPositions"
    GET_ACCOUNT_INSTRUMENT_STATISTICS = "GetAccountInstrumentStatistics"
    GET_ACCOUNT_INFO = "GetAccountInfo"

    # authentication
    AUTHENTICATE_USER = "AuthenticateUser"
    WEB_AUTHENTICATE_USER = "WebAuthenticateUser"
    LOGOUT = "LogOut"

    # # fee
    GET_WITHDRAW_FEE = "GetWithdrawFee"
    GET_DEPOSIT_FEE = "GetDepositFee"
    GET_OMS_DEPOSIT_FEES = "GetOMSDepositFees"
    GET_OMS_WITHDRAW_FEES = "GetOMSWithdrawFees"
    GET_ACCOUNT_FEES = "GetAccountFees"
    GET_ORDER_FEE = "GetOrderFee"

    # instrument
    GET_INSTRUMENT = "GetInstrument"
    GET_INSTRUMENTS = "GetInstruments"
    GET_INSTRUMENT_VERIFICATION_LEVEL_CONFIG = "GetInstrumentVerificationLevelConfig"

    # product
    GET_PRODUCT = "GetProduct"
    GET_PRODUCTS = "GetProducts"
    GET_VERIFICATION_LEVEL_CONFIG = "GetVerificationLevelConfig"

    # reports
    GENERATE_TRADE_ACTIVITY_REPORT = "GenerateTradeActivityReport"
    GENERATE_TRANSACTION_ACTIVITY_REPORT = "GenerateTransactionActivityReport"
    GENERATE_PRODUCT_DELTA_ACTIVITY_REPORT = "GenerateProductDeltaActivityReport"
    GENERATE_PNL_ACTIVITY_REPORT = "GeneratePnLActivityReport"
    SCHEDULE_TRADE_ACTIVITY_REPORT = "ScheduleTradeActivityReport"
    SCHEDULE_TRANSACTION_ACTIVITY_REPORT = "ScheduleTransactionActivityReport"
    SCHEDULE_PRODUCT_DELTA_ACTIVITY_REPORT = "ScheduleProductDeltaActivityReport"
    SCHEDULE_PROFIT_AND_LOSS_ACTIVITY_REPORT = "ScheduleProfitAndLossActivityReport"
    CANCEL_USER_REPORT = "CancelUserReport"
    GET_USER_REPORT_WRITER_RESULT_RECORDS = "GetUserReportWriterResultRecords"
    GET_USER_REPORT_TICKETS = "GetUserReportTickets"
    REMOVE_USER_REPORT_TICKET = "RemoveUserReportTicket"
    GET_USER_REPORT_TICKETS_BY_STATUS = "GetUserReportTicketsByStatus"
    DOWNLOAD_DOCUMENT = "DownloadDocument"
    DOWNLOAD_DOCUMENT_SLICE = "DownloadDocumentSlice"

    # ping
    PING = "Ping"
    HEALTH_CHECK = "HealthCheck"

    # trading
    SEND_ORDER_LIST = "SendOrderList"
    SEND_CANCEL_LIST = "SendCancelList"
    SEND_CANCEL_REPLACE_LIST = "SendCancelReplaceList"
    MODIFY_ORDER = "ModifyOrder"
    CANCEL_ALL_ORDERS = "CancelAllOrders"
    GET_ORDER_STATUS = "GetOrderStatus"
    GET_ORDERS_HISTORY = "GetOrdersHistory"
    GET_TRADES_HISTORY = "GetTradesHistory"
    GET_ORDER_HISTORY_BY_ORDER_ID = "GetOrderHistoryByOrderId"
    GET_TICKER_HISTORY = "GetTickerHistory"
    GET_LAST_TRADES = "GetLastTrades"
    GET_LEVEL1_SUMMARY = "GetLevel1Summary"
    GET_LEVEL1_SUMMARY_MIN = "GetLevel1SummaryMin"
    GET_OPEN_TRADE_REPORTS = "GetOpenTradeReports"
    GET_ORDERS = "GetOrders"
    GET_ORDER_HISTORY = "GetOrderHistory"
    SEND_ORDER = "SendOrder"
    CANCEL_REPLACE_ORDER = "CancelReplaceOrder"
    CANCEL_ORDER = "CancelOrder"
    GET_OPEN_ORDERS = "GetOpenOrders"
    GET_ACCOUNT_TRADES = "GetAccountTrades"
    SUMMARY = "Summary"
    TICKER = "Ticker"
    ORDER_BOOK = "OrderBook"
    TRADES = "Trades"
    GET_L2_SNAPSHOT = "GetL2Snapshot"
    GET_LEVEL1 = "GetLevel1"
    GET_ENUMS = "GetEnums"

    # user
    GET_USER_ACCOUNTS = "GetUserAccounts"
    GET_USER_DEVICES = "GetUserDevices"
    GET_USER_INFO = "GetUserInfo"
    GET_USER_PERMISSIONS = "GetUserPermissions"

    # wallet
    BANKS = "banks"
    BANK_ACCOUNTS = "bank-accounts"
    GET_NETWORK_TEMPLATES = "wallet/crypto/withdrawal/templates"
    GET_DEPOSIT_ADDRESSES = "wallet/crypto"
    CREATE_DEPOSIT_ADDRESS = "wallet/crypto"
    WHITELISTED_ADDRESSES = "wallet/crypto/whitelist-addresses"
    UPDATE_ONE_STEP_WITHDRAW = "wallet/crypto/whitelist-addresses/one-step/status"
    CREATE_CRIPTO_WITHDRAW = "wallet/crypto/withdrawal"
    FIAT_DEPOSIT = "wallet/fiat/deposit"
    GET_OWNERS_FIAT_WITHDRAW = "wallet/fiat/withdrawal/owners"
    FIAT_WITHDRAW = "wallet/fiat/withdrawal"
    TRANSFER_FUNDS = "wallet/transfer-funds"
    GET_TRANSACTIONS = "wallet/transactions"

    # quote
    QUOTES = "quotes"
    QUOTES_DIRECT = "quotes/direct"
    QUOTES_INVERSE = "quotes/inverse"


class WebSocketEndpoint(str, Enum):
    SUBSCRIBE_LEVEL2 = "SubscribeLevel2"
    UNSUBSCRIBE_LEVEL2 = "UnsubscribeLevel2"
    SUBSCRIBE_LEVEL1 = "SubscribeLevel1"
    UNSUBSCRIBE_LEVEL1 = "UnsubscribeLevel1"
    SUBSCRIBE_TRADES = "SubscribeTrades"
    UPDATE_TRADES = "TradeDataUpdateEvent"
    UNSUBSCRIBE_TRADES = "UnsubscribeTrades"
    SUBSCRIBE_TICKER = "SubscribeTicker"
    UNSUBSCRIBE_TICKER = "UnsubscribeTicker"
    SUBSCRIBE_ACCOUNT_EVENTS = "SubscribeAccountEvents"
    UNSUBSCRIBE_ACCOUNT_EVENTS = "UnSubscribeAccountEvents"
    SUBSCRIBE_ORDER_STATE_EVENTS = "SubscribeOrderStateEvents"
    UNSUBSCRIBE_ORDER_STATE_EVENTS = "UnSubscribeOrderStateEvents"

    WEB_AUTHENTICATE_USER = "WebAuthenticateUser"
    ACCOUNT_EVENT_TRANSACTION = "TransactionEvent"
    ACCOUNT_EVENT_WITHDRAW_TICKET_UPDATE = "WithdrawTicketUpdateEvent"
    ACCOUNT_EVENT_ACCOUNT_POSITION = "AccountPositionEvent"
    ACCOUNT_EVENT_ORDER_TRADE = "OrderTradeEvent"
    ACCOUNT_EVENT_ORDER_STATE = "OrderStateEvent"
    ACCOUNT_EVENT_DEPOSIT_TICKET_UPDATE = "DepositTicketUpdateEvent"
    ACCOUNT_EVENT_ACCOUNT_INFO_UPDATE = "AccountInfoUpdateEvent"
    ACCOUNT_EVENT_CANCEL_ORDER_REJECT = "CancelOrderRejectEvent"
    ACCOUNT_EVENT_DEPOSIT = "DepositEvent"
    UPDATE_TICKER = "TickerDataUpdateEvent"
    SUBSCRIBE_LEVEL_1 = "SubscribeLevel1"
    UPDATE_LEVEL_1 = "Level1UpdateEvent"
    SUBSCRIBE_LEVEL_2 = "SubscribeLevel2"
    UPDATE_LEVEL_2 = "Level2UpdateEvent"



from dataclasses import dataclass
from typing import List
from uuid import UUID


@dataclass
class Bank:
    id: UUID
    name: str
    country: str


@dataclass
class Banks:
    total: int
    data: List[Bank]



from dataclasses import dataclass, field
from decimal import Decimal
from typing import List, Optional


@dataclass
class WithdrawTicket:
    oms_id: int
    account_id: int
    asset_manager_id: Optional[int] = None
    account_provider_id: Optional[int] = None
    account_name: Optional[str] = None
    asset_id: Optional[int] = None
    asset_name: Optional[str] = None
    amount: Optional[Decimal] = None
    notional_value: Optional[Decimal] = None
    notional_product_id: Optional[int] = None
    template_form: Optional[str] = None
    template_form_type: Optional[str] = None
    request_code: Optional[str] = None
    request_ip: Optional[str] = None
    request_user_id: Optional[int] = None
    request_user_name: Optional[str] = None
    operator_id: Optional[int] = None
    status: Optional[int] = None
    fee_amt: Optional[Decimal] = None
    updated_by_user: Optional[int] = None
    updated_by_user_name: Optional[str] = None
    ticket_number: Optional[int] = None
    withdraw_transaction_details: Optional[str] = None
    reject_reason: Optional[str] = None
    created_time_stamp: Optional[str] = None
    last_update_time_stamp: Optional[str] = None
    created_timestamp_tick: Optional[int] = None
    last_update_timestamp_tick: Optional[int] = None
    comments: List[str] = field(default_factory=list)
    attachments: List[str] = field(default_factory=list)
    audit_log: List[str] = field(default_factory=list)



from dataclasses import dataclass
from typing import Optional


@dataclass
class NetworkTemplate:
    name: str
    type: str
    required: bool
    max_length: Optional[int] = None
    min_length: Optional[int] = None



from dataclasses import dataclass
from uuid import UUID


@dataclass
class ConfirmFiatWithdrawRequest:
    withdrawal_id: UUID
    attempt_code: str


@dataclass
class ConfirmFiatWithdrawRequestInternal:
    attempt_code: str



from dataclasses import dataclass
from decimal import Decimal
from typing import Optional, Union

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


@dataclass
class SetOmsWithdrawFeeRequest(WithOMSId):
    product_id: int
    account_id: Optional[int] = None
    account_provider_id: Optional[int] = None
    fee_amt: Optional[Decimal] = None
    fee_calc_type: Optional[Union[str, int]] = None
    is_active: Optional[bool] = None
    minimal_fee_amt: Optional[Decimal] = None
    minimal_fee_calc_type: Optional[Union[str, int]] = None



import unittest
from notbank_python_sdk.notbank_client import NotbankClient


from notbank_python_sdk.requests_models.remove_user_report_ticket import RemoveUserReportTicketRequest
from tests import test_helper


class TestRemoveUserReportTicket(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_remove_user_report_ticket_success(self):
        req = RemoveUserReportTicketRequest(
            "e27e5268-db50-70fa-de84-ee0b6ae16093")
        self.client.remove_user_report_ticket(req)

    def test_remove_user_report_ticket_failure(self):
        req = RemoveUserReportTicketRequest(
            '{"user_report_ticket_id": "bad-format"}')
        self.client.remove_user_report_ticket(req)


if __name__ == "__main__":
    unittest.main()



from typing import Callable,  Optional

from notbank_python_sdk.core.converter import from_json_str
from notbank_python_sdk.error import StandardErrorResponse
from notbank_python_sdk.websocket.callback_identifier import CallbackIdentifier
from notbank_python_sdk.websocket.message_frame import MessageFrame
from notbank_python_sdk.websocket.callback_manager import CallbackManager
from notbank_python_sdk.websocket.message_type import MessageType

MessageFrameCallback = Callable[[MessageFrame], None]


class WebsocketResponseHandler:
    _get_sequenced_callback: Callable[[int], Optional[MessageFrameCallback]]
    _get_subscription_callback: Callable[[
        str], Optional[Callable[[str], None]]]
    _on_failure: Callable[[Exception], None]

    def __init__(
        self,
        get_sequenced_callback: Callable[[int], Optional[MessageFrameCallback]],
        get_subscription_callback: Callable[[str], Optional[Callable[[str], None]]],
        on_failure: Callable[[Exception], None]
    ):
        self._get_sequenced_callback = get_sequenced_callback
        self._get_subscription_callback = get_subscription_callback
        self._on_failure = on_failure

    @staticmethod
    def create(callback_manager: CallbackManager, on_failure: Callable[[Exception], None]) -> 'WebsocketResponseHandler':
        return WebsocketResponseHandler(
            callback_manager.sequenced_callbacks.pop,
            callback_manager.subscription_callbacks.get,
            on_failure)

    def handle(self, message: str) -> None:
        try:
            message_frame = from_json_str(MessageFrame, message)
            if message_frame.is_left():
                self._on_failure(message_frame.get_left())
                return
            callback = self._get_sequenced_callback(message_frame.get().i)
            if callback:
                callback(message_frame.get())
                if self.is_error_message(message_frame.get()):
                    return
            callback_identifier = CallbackIdentifier.get_from_message_frame(
                message_frame.get())
            if callback_identifier is None:
                return
            subscription_callback = self._get_subscription_callback(
                callback_identifier)
            if subscription_callback is not None:
                subscription_callback(message_frame.get().o)
        except Exception as e:
            self._on_failure(e)

    def is_error_message(self, message_frame: MessageFrame) -> bool:
        is_error_type = message_frame.m == MessageType.ERROR
        if is_error_type:
            return True
        standard_error = from_json_str(StandardErrorResponse, message_frame.o)
        if standard_error.is_left():
            return False
        return standard_error.get().result is not None and not standard_error.get().result == False



import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_user_report_writer_result_records import GetUserReportWriterResultRecordsRequest
from tests import test_helper


class TestGetUserReportWriterResultRecords(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_user_report_writer_result_records_success(self):
        req = GetUserReportWriterResultRecordsRequest(user_id=6)
        result = self.client.get_user_report_writer_result_records(req)
        self.assertIsInstance(result, list)
        self.assertEqual(len(result), 1)
        r = result[0]
        self.assertEqual(r.requesting_user, 6)
        self.assertEqual(r.result_status, "SuccessComplete")
        self.assertTrue("TradeActivity" in r.report_descriptive_header)

    def test_get_user_report_writer_result_records_no_results(self):
        req = GetUserReportWriterResultRecordsRequest(user_id=99)
        result = self.client.get_user_report_writer_result_records(req)
        self.assertIsInstance(result, list)
        self.assertEqual(result, [])


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from decimal import Decimal


@dataclass
class InstrumentSummary:
    trading_pairs: str
    last_price: Decimal
    lowest_ask: Decimal
    highest_bid: Decimal
    base_volume: Decimal
    quote_volume: Decimal
    price_change_percent_24h: Decimal
    highest_price_24h: Decimal
    lowest_price_24h: Decimal



from dataclasses import dataclass
from typing import Optional

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


@dataclass
class UnsubscribeLevel2Request(WithOMSId):
    instrument_id: Optional[int] = None
    symbol: Optional[str] = None



from decimal import Decimal
from typing import Any, List
from notbank_python_sdk.core.either import Either

from notbank_python_sdk.error import ErrorCode, NotbankException
from dataclasses import dataclass

import simplejson as json


@dataclass
class Level2Feed:
    market_data_update_id: int
    number_of_accounts: int
    action_date_time: int
    action_type: int
    last_trade_price: Decimal
    number_of_orders: int
    price: Decimal
    product_pair_code: int
    quantity: Decimal
    side: int


def level2_ticker_feed_from_list(json_list: List[Any]) -> Level2Feed:
    return Level2Feed(
        market_data_update_id=json_list[0],
        number_of_accounts=json_list[1],
        action_date_time=json_list[2],
        action_type=json_list[3],
        last_trade_price=Decimal(json_list[4]),
        number_of_orders=json_list[5],
        price=Decimal(json_list[6]),
        product_pair_code=json_list[7],
        quantity=Decimal(json_list[8]),
        side=json_list[9]
    )


def level_2_ticker_feed_list_from_json_str(json_str: str) -> Either[NotbankException, List[Level2Feed]]:
    try:
        json_list = json.loads(json_str, use_decimal=True)
        return Either.right([level2_ticker_feed_from_list(item) for item in json_list])
    except Exception as e:
        return Either.left(NotbankException(ErrorCode.CONFIGURATION_ERROR, str(e)))



import unittest
from uuid import UUID
from notbank_python_sdk.requests_models.confirm_whitelisted_address_request import ConfirmWhiteListedAddressRequest
from notbank_python_sdk.requests_models.delete_whitelisted_address_request import DeleteWhiteListedAddressRequest

from tests import test_helper

from notbank_python_sdk.notbank_client import NotbankClient


class TestDeleteWhitelistedAddress(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection(
            test_helper.print_message_in, test_helper.print_message_out)
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_delete_whitelisted_address(self):
        response = self.client.delete_whitelisted_address(DeleteWhiteListedAddressRequest(
            whitelisted_address_id=UUID(
                "3b84035d-c015-4646-bdee-bc0f500402cc"),
            account_id=self.credentials.account_id,
            otp="125814"))
        self.assertIsNone(response)


if __name__ == "__main__":
    unittest.main()



from unittest import TestCase, main

from notbank_python_sdk.error import NotbankException
from notbank_python_sdk.requests_models.account_fees_request import AccountFeesRequest
from notbank_python_sdk.requests_models.authenticate_request import AuthenticateRequest
from notbank_python_sdk.requests_models.get_account_trades import GetAccountTradesRequest
from notbank_python_sdk.notbank_client import NotbankClient
from tests.test_helper import load_credentials, new_websocket_client_connection, new_rest_client_connection


class AuthenticatedMethodTestCase(TestCase):
    def test_rest_authentication(self):
        rest_client_connection = new_rest_client_connection()
        system_client = NotbankClient(rest_client_connection)
        credentials = load_credentials('keys.json')
        auth_response = system_client.authenticate(AuthenticateRequest(
            api_public_key=credentials.public_key,
            api_secret_key=credentials.secret_key,
            user_id=credentials.user_id,
        ))
        try:
            fees = system_client.get_account_fees(AccountFeesRequest(9))
        except NotbankException as e:
            print(f"error: {e}")

    def test_websocket_authentication(self):
        websocket_client_connection = new_websocket_client_connection()
        websocket_client_connection.connect()
        system_client = NotbankClient(websocket_client_connection)
        credentials = load_credentials('keys.json')
        auth_response = system_client.authenticate(AuthenticateRequest(
            api_public_key=credentials.public_key,
            api_secret_key=credentials.secret_key,
            user_id=credentials.user_id,
        ))
        try:
            account_trades = system_client.get_account_trades(
                GetAccountTradesRequest(1, credentials.account_id))
        except NotbankException as e:
            print(f"error: {e}")
        websocket_client_connection.close()


if __name__ == '__main__':
    main()



from decimal import Decimal
import unittest

from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_order_fee_request import GetOrderFeeRequest
from tests import test_helper


class TestGetOrderFee(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_order_fee_success(self):
        """
        Prueba exitosa: Solicitud válida, devuelve la estimación de la comisión.
        """
        request = GetOrderFeeRequest(
            account_id=9,
            instrument_id=1,
            quantity=Decimal(0.5),
            price=Decimal(10000.0),
            order_type=2,
            maker_taker=1,
            side=0,
        )
        response = self.client.get_order_fee(request)

        # Verificaciones
        self.assertIsNotNone(response)
        self.assertEqual(response.order_fee, 0.00001)
        self.assertEqual(response.product_id, 2)

    def test_get_order_fee_not_found(self):
        """
        Prueba: Solicitud inválida, no se encuentra la estimación.
        """
        request = GetOrderFeeRequest(
            account_id=999,  # account_id inválido
            instrument_id=1,
            quantity=Decimal(0.5),
            price=Decimal(10000.0),
            order_type=2,
            maker_taker=1,
            side=0,
        )
        response = self.client.get_order_fee(request)

        # Verificaciones
        self.assertIsNone(response)


if __name__ == "__main__":
    unittest.main()



import secrets
from hmac import HMAC
from hashlib import sha256

from notbank_python_sdk.requests_models.authenticate_request import AuthenticateRequestData, AuthenticateRequest


class Authenticator:
    @staticmethod
    def _new_nonce() -> str:
        return str(secrets.randbelow(10_000_000_00))

    @staticmethod
    def _get_request_signature(authenticate_request: AuthenticateRequest, nonce: str) -> str:
        message = nonce+str(authenticate_request.user_id) + \
            authenticate_request.api_public_key
        return HMAC(key=authenticate_request.api_secret_key.encode(),
                    msg=message.encode(),
                    digestmod=sha256).hexdigest()

    @classmethod
    def convert_data(cls, authenticate_request: AuthenticateRequest) -> AuthenticateRequestData:
        nonce = cls._new_nonce()
        signature = cls._get_request_signature(authenticate_request, nonce)
        return AuthenticateRequestData(
            api_key=authenticate_request.api_public_key,
            signature=signature,
            user_id=str(authenticate_request.user_id),
            nonce=nonce
        )



from notbank_python_sdk.requests_models.account_fees_request import *
from notbank_python_sdk.requests_models.add_whitelisted_address_request import *
from notbank_python_sdk.requests_models.authenticate_request import *
from notbank_python_sdk.requests_models.cancel_all_orders import *
from notbank_python_sdk.requests_models.cancel_order_request import *
from notbank_python_sdk.requests_models.cancel_order import *
from notbank_python_sdk.requests_models.cancel_replace_order_request import *
from notbank_python_sdk.requests_models.cancel_user_report import *
from notbank_python_sdk.requests_models.confirm_whitelisted_address_request import *
from notbank_python_sdk.requests_models.create_fiat_deposit_request import *
from notbank_python_sdk.requests_models.create_fiat_withdraw_request import *
from notbank_python_sdk.requests_models.add_client_bank_account_request import *
from notbank_python_sdk.requests_models.create_crypto_withdraw_request import *
from notbank_python_sdk.requests_models.delete_client_bank_account_request import *
from notbank_python_sdk.requests_models.delete_whitelisted_address_request import *
from notbank_python_sdk.requests_models.deposit_address_request import *
from notbank_python_sdk.requests_models.download_document_slice import *
from notbank_python_sdk.requests_models.download_document import *
from notbank_python_sdk.requests_models.enums import *
from notbank_python_sdk.requests_models.fee_request import *
from notbank_python_sdk.requests_models.generate_pnl_activity_report import *
from notbank_python_sdk.requests_models.generate_product_delta_activity_report import *
from notbank_python_sdk.requests_models.generate_trade_activity_report import *
from notbank_python_sdk.requests_models.generate_transaction_activity_report import *
from notbank_python_sdk.requests_models.get_account_info_request import *
from notbank_python_sdk.requests_models.get_account_instrument_statistics_request import *
from notbank_python_sdk.requests_models.get_account_positions_request import *
from notbank_python_sdk.requests_models.get_account_trades import *
from notbank_python_sdk.requests_models.get_account_transactions_request import *
from notbank_python_sdk.requests_models.get_client_bank_account_request import *
from notbank_python_sdk.requests_models.get_client_bank_accounts_request import *
from notbank_python_sdk.requests_models.get_banks_request import *
from notbank_python_sdk.requests_models.get_deposit_fee_request import *
from notbank_python_sdk.requests_models.get_instrument_request import *
from notbank_python_sdk.requests_models.get_instruments_request import *
from notbank_python_sdk.requests_models.get_l2_snapshot import *
from notbank_python_sdk.requests_models.get_last_trades import *
from notbank_python_sdk.requests_models.get_level1_summary_min import *
from notbank_python_sdk.requests_models.get_level1_summary import *
from notbank_python_sdk.requests_models.get_network_templates_request import *
from notbank_python_sdk.requests_models.get_open_orders import *
from notbank_python_sdk.requests_models.get_open_trade_reports import *
from notbank_python_sdk.requests_models.get_order_fee_request import *
from notbank_python_sdk.requests_models.get_order_history_by_order_id import *
from notbank_python_sdk.requests_models.get_order_history import *
from notbank_python_sdk.requests_models.get_order_status import *
from notbank_python_sdk.requests_models.get_orders_history import *
from notbank_python_sdk.requests_models.get_orders import *
from notbank_python_sdk.requests_models.get_product_request import *
from notbank_python_sdk.requests_models.get_products_request import *
from notbank_python_sdk.requests_models.get_ticker_history import *
from notbank_python_sdk.requests_models.get_trades_history import *
from notbank_python_sdk.requests_models.get_user_info import *
from notbank_python_sdk.requests_models.get_user_permissions import *
from notbank_python_sdk.requests_models.get_user_report_tickets_by_status import *
from notbank_python_sdk.requests_models.get_user_report_tickets import *
from notbank_python_sdk.requests_models.get_user_report_writer_result_records import *
from notbank_python_sdk.requests_models.get_whitelisted_addresses_request import *
from notbank_python_sdk.requests_models.level1 import *
from notbank_python_sdk.requests_models.modify_order import *
from notbank_python_sdk.requests_models.oms_fees_request import *
from notbank_python_sdk.requests_models.order_book import *
from notbank_python_sdk.requests_models.remove_user_report_ticket import *
from notbank_python_sdk.requests_models.schedule_product_delta_activity_report import *
from notbank_python_sdk.requests_models.schedule_profit_and_loss_activity_report import *
from notbank_python_sdk.requests_models.schedule_trade_activity_report import *
from notbank_python_sdk.requests_models.schedule_transaction_activity_report import *
from notbank_python_sdk.requests_models.send_order import *
from notbank_python_sdk.requests_models.set_oms_withdraw_fee import *
from notbank_python_sdk.requests_models.subscribe_account_events_request import *
from notbank_python_sdk.requests_models.subscribe_level1_request import *
from notbank_python_sdk.requests_models.subscribe_level2_request import *
from notbank_python_sdk.requests_models.subscribe_order_state_events_request import *
from notbank_python_sdk.requests_models.subscribe_ticker_request import *
from notbank_python_sdk.requests_models.subscribe_trades_request import *
from notbank_python_sdk.requests_models.trades import *
from notbank_python_sdk.requests_models.unsubscribe_account_events_request import *
from notbank_python_sdk.requests_models.unsubscribe_level1_request import *
from notbank_python_sdk.requests_models.unsubscribe_level2_request import *
from notbank_python_sdk.requests_models.unsubscribe_order_state_events_request import *
from notbank_python_sdk.requests_models.unsubscribe_ticker_request import *
from notbank_python_sdk.requests_models.unsubscribe_trades_request import *
from notbank_python_sdk.requests_models.update_one_step_withdraw_request import *
from notbank_python_sdk.requests_models.user_accounts import *
from notbank_python_sdk.requests_models.user_devices import *
from notbank_python_sdk.requests_models.verification_level_config_request import *
from notbank_python_sdk.requests_models.web_authenticate_user_request import *
from notbank_python_sdk.requests_models.with_oms_id import *
from notbank_python_sdk.requests_models.create_inverse_quote_request import *
from notbank_python_sdk.requests_models.resend_verification_code_whitelisted_address_request import *
from notbank_python_sdk.requests_models.get_owners_fiat_withdraw import *
from notbank_python_sdk.requests_models.get_quote_request import *
from notbank_python_sdk.requests_models.transfer_funds_request import *
from notbank_python_sdk.requests_models.get_quotes_request import *
from notbank_python_sdk.requests_models.create_direct_quote_request import *
from notbank_python_sdk.requests_models.get_transactions_request import *
from notbank_python_sdk.requests_models.execute_quote_request import *
from notbank_python_sdk.requests_models.confirm_fiat_withdraw_request import *



import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_user_permissions import GetUserPermissionsRequest
from tests import test_helper


class TestGetUserPermissions(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_user_permissions_success(self):
        req = GetUserPermissionsRequest(user_id=6)
        res = self.client.get_user_permissions(req)
        self.assertEqual(res, [
                         "Deposit", "Operator", "Trading", "Withdraw"])

    def test_get_user_permissions_empty(self):
        req = GetUserPermissionsRequest(user_id=999)
        res = self.client.get_user_permissions(req)
        self.assertEqual(res, [])


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from enum import Enum
from typing import Optional


class ErrorCode(Enum):
    TIMED_OUT = -300
    CONFIGURATION_ERROR = -200
    INVALID_RESPONSE = -100
    UNKNOWN = -1
    SUCCESS = 0
    NOT_AUTHORIZED = 20
    INVALID_REQUEST = 100
    OPERATION_FAILED = 101
    SERVER_ERROR = 102
    RESOURCE_NOT_FOUND = 104


@dataclass
class StandardErrorResponse:
    result: bool
    errormsg: Optional[str] = None
    errorcode: ErrorCode = ErrorCode.SUCCESS
    detail: Optional[str] = None


def _get_message(standard_error_response: StandardErrorResponse) -> str:
    message = standard_error_response.errormsg or ""
    detail = standard_error_response.detail or ""
    return message + ". " + detail


class NotbankException(Exception):
    def __init__(self, code: ErrorCode, message: str):
        self.code = code
        super().__init__(message)

    @staticmethod
    def create(standard_error_response: StandardErrorResponse) -> 'NotbankException':
        message = _get_message(standard_error_response)
        return NotbankException(standard_error_response.errorcode, message)



from dataclasses import dataclass
from decimal import Decimal


@dataclass
class Instrument:
    oms_id: int
    instrument_id: int
    symbol: str
    product1: int
    product1_symbol: str
    product2: int
    product2_symbol: str
    instrument_type: str
    venue_instrument_id: int
    venue_id: int
    sort_index: int
    session_status: str
    previous_session_status: str
    session_status_date_time: str
    self_trade_prevention: bool
    quantity_increment: Decimal
    price_increment: Decimal
    minimum_quantity: Decimal
    minimum_price: Decimal
    venue_symbol: str
    is_disable: bool
    master_data_id: int
    price_collar_threshold: Decimal
    price_collar_percent: Decimal
    price_collar_enabled: bool
    price_floor_limit: Decimal
    price_floor_limit_enabled: bool
    price_ceiling_limit: Decimal
    price_ceiling_limit_enabled: bool
    create_with_market_running: bool
    allow_only_market_maker_counter_party: bool
    price_collar_index_difference: Decimal
    price_collar_convert_to_otc_enabled: bool
    price_collar_convert_to_otc_client_user_id: int
    price_collar_convert_to_otc_account_id: int
    price_collar_convert_to_otc_threshold: Decimal
    otc_convert_size_enabled: bool
    otc_convert_size_threshold: Decimal
    otc_trades_public: bool
    price_tier: int



from dataclasses import dataclass
from typing import Optional

from notbank_python_sdk.models.user import User as AUser


@dataclass
class AuthenticateResponse:
    authenticated: bool
    requires_2fa: Optional[bool] = None
    two_fa_type: Optional[str] = None
    two_fa_token: Optional[str] = None
    locked: Optional[bool] = None
    user: Optional[AUser] = None
    errormsg: Optional[str] = None
    session_token: Optional[str] = None
    enforce_enable_2fa: Optional[bool] = None
    pending2_fa_token: Optional[str] = None



from dataclasses import dataclass
from decimal import Decimal


@dataclass
class Product:
    oms_id: int
    product_id: int
    product: str
    product_full_name: str
    master_data_unique_product_symbol: str
    product_type: str
    decimal_places: int
    tick_size: Decimal
    deposit_enabled: bool
    withdraw_enabled: bool
    no_fees: bool
    is_disabled: bool
    margin_enabled: bool



import unittest

from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.models.account_transaction import (
    TransactionType,
    TransactionReferenceType,
)
from notbank_python_sdk.requests_models.get_account_transactions_request import (
    GetAccountTransactionsRequest,
)
from tests import test_helper


class TestGetAccountTransactions(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_transactions_success(self):
        """Prueba exitosa: obtención de transacciones válidas."""
        request = GetAccountTransactionsRequest(
            account_id=7, depth=2, product_id=3
        )
        response = self.client.get_account_transactions(request)

        self.assertEqual(len(response), 2)
        self.assertEqual(response[0].transaction_id, 24214)
        self.assertEqual(response[0].cr, 0.01247667)
        self.assertEqual(response[0].dr, 0.0)
        self.assertEqual(response[0].transaction_type, TransactionType.trade)
        self.assertEqual(response[0].reference_type,
                         TransactionReferenceType.trade)
        self.assertEqual(response[1].transaction_id, 24215)
        self.assertEqual(response[1].transaction_type, TransactionType.trade)

    def test_no_transactions_found(self):
        """Prueba: No se encuentran transacciones en la cuenta."""
        request = GetAccountTransactionsRequest(account_id=999)
        response = self.client.get_account_transactions(request)
        self.assertEqual(len(response), 0)

    def test_invalid_omsid(self):
        """Prueba fallida: oms_id inválido."""
        request = GetAccountTransactionsRequest(account_id=7)
        with self.assertRaises(ValueError):
            self.client.get_account_transactions(request)


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from typing import Optional

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


@dataclass
class GetOrderHistoryRequest(WithOMSId):
    account_id: int
    order_state: Optional[str] = None
    order_id: Optional[int] = None
    client_order_id: Optional[int] = None
    original_order_id: Optional[int] = None
    original_client_order_id: Optional[int] = None
    user_id: Optional[int] = None
    instrument_id: Optional[int] = None
    start_timestamp: Optional[int] = None
    end_timestamp: Optional[int] = None
    depth: Optional[int] = None
    limit: Optional[int] = None
    start_index: Optional[int] = None



import unittest

from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_order_history_by_order_id import GetOrderHistoryByOrderIdRequest
from tests import test_helper


class TestGetOrderHistoryByOrderId(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_order_history_by_order_id_success(self):
        """Prueba exitosa: order_id válido."""
        request = GetOrderHistoryByOrderIdRequest(
            order_id=6459,
        )
        response = self.client.get_order_history_by_order_id(request)

        # Verificar que devuelve correctamente 1 estado de orden
        self.assertEqual(len(response), 1)
        self.assertEqual(response[0].order_id, 6459)
        self.assertEqual(response[0].account_name, "sample_user")
        self.assertEqual(response[0].order_state, "FullyExecuted")
        self.assertEqual(response[0].quantity_executed, 18307.63)

    def test_get_order_history_by_order_id_not_found(self):
        """Prueba: order_id inválido, no se encuentra el registro."""
        request = GetOrderHistoryByOrderIdRequest(
            order_id=999,
        )
        response = self.client.get_order_history_by_order_id(request)

        # Verificar que no se devuelven estados de orden
        self.assertEqual(len(response), 0)


# Punto de entrada para ejecutar pruebas
if __name__ == "__main__":
    unittest.main()



from typing import Any, Callable, TypeVar
from notbank_python_sdk.core.endpoint_category import EndpointCategory
from notbank_python_sdk.models.authenticate_response import AuthenticateResponse
from notbank_python_sdk.requests_models.authenticate_request import AuthenticateRequest
from notbank_python_sdk.websocket.subscription import Subscription, Unsubscription
from notbank_python_sdk.websocket.websocket_restarter.restarter import Restarter

T = TypeVar("T")


class RestartingWebsocketConnection:
    _restarter: Restarter

    def __init__(self, restarter: Restarter):
        self._restarter = restarter

    def connect(self) -> None:
        self._restarter.reconnect()

    def close(self) -> None:
        self._restarter.close()

    def subscribe(self, subscription: Subscription[T]) -> T:
        subscription_result = self._restarter.get_connection().subscribe(subscription)
        self._restarter.get_resubscriber().save(subscription)
        return subscription_result

    def unsubscribe(self, unsubscription: Unsubscription[T]) -> T:
        unsubscription_result = self._restarter.get_connection().unsubscribe(unsubscription)
        self._restarter.get_resubscriber().remove(unsubscription)
        return unsubscription_result

    def request(
            self,
            endpoint: str,
            endpoint_category: EndpointCategory,
            request_message: Any,
            parse_response_fn: Callable[[Any], T]) -> T:
        return self._restarter.get_connection().request(endpoint, endpoint_category, request_message, parse_response_fn)

    def authenticate_user(self, authenticate_request: AuthenticateRequest) -> AuthenticateResponse:
        authentication_response = self._restarter.get_connection(
        ).authenticate_user(authenticate_request)
        if authentication_response.authenticated:
            self._restarter.get_reauther().update_authenticate_fn(
                lambda connection: connection.authenticate_user(authenticate_request))
        return authentication_response



from typing import Any, Callable, NamedTuple, Optional


class ConnectionConfiguration(NamedTuple):
    uri: str
    on_open: Callable[[], None] = lambda: None
    on_close: Callable[[Any, str], None] = lambda code, message: None
    on_failure: Callable[[Exception], None] = lambda e: None
    peek_message_in: Callable[[str], None] = lambda x: None
    peek_message_out: Callable[[str], None] = lambda x: None
    request_timeout: Optional[float] = None



from dataclasses import dataclass
from decimal import Decimal

import simplejson as json
from typing import Any, List
from notbank_python_sdk.core.either import Either

from notbank_python_sdk.error import ErrorCode, NotbankException


@dataclass
class Ticker:
    end_date_time: int
    high: Decimal
    low: Decimal
    open: Decimal
    close: Decimal
    volume: Decimal
    bid: Decimal
    ask: Decimal
    instrument_id: int
    begin_date_time: int


def ticker_from_list(json_list: List[Any]) -> Ticker:
    return Ticker(
        end_date_time=json_list[0],
        high=Decimal(json_list[1]),
        low=Decimal(json_list[2]),
        open=Decimal(json_list[3]),
        close=Decimal(json_list[4]),
        volume=Decimal(json_list[5]),
        bid=Decimal(json_list[6]),
        ask=Decimal(json_list[7]),
        instrument_id=json_list[8],
        begin_date_time=json_list[9],
    )


def ticker_list_from_json_str(json_str: str) -> Either[NotbankException, List[Ticker]]:
    try:
        json_list = json.loads(json_str, use_decimal=True)
        return Either.right([ticker_from_list(item) for item in json_list])
    except Exception as e:
        return Either.left(NotbankException(ErrorCode.CONFIGURATION_ERROR, str(e)))



import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_account_positions_request import GetAccountPositionsRequest
from tests import test_helper


class TestGetAccountPositions(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_account_positions_success(self):
        request = GetAccountPositionsRequest(
            account_id=self.credentials.account_id)
        account_positions = self.client.get_account_positions(request)
            

# Punto de entrada para ejecutar pruebas
if __name__ == "__main__":
    unittest.main()
